package uoc.edu.client.layout;

import java.util.ArrayList;
import java.util.List;

import org.gwtopenmaps.openlayers.client.feature.VectorFeature;
import org.gwtopenmaps.openlayers.client.util.Attributes;

import uoc.edu.client.openlayers.UpdateState;
import uoc.edu.client.service.ComarcaService;
import uoc.edu.client.service.ComarcaServiceAsync;
import uoc.edu.client.service.EsdevenimentService;
import uoc.edu.client.service.EsdevenimentServiceAsync;
import uoc.edu.client.service.PoblacioService;
import uoc.edu.client.service.PoblacioServiceAsync;
import uoc.edu.client.service.ServiceConstants;
import uoc.edu.client.service.TipusEsdevenimentService;
import uoc.edu.client.service.TipusEsdevenimentServiceAsync;
import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.TipusEsdeveniment;
import uoc.edu.shared.utils.DateUtils;

import com.extjs.gxt.ui.client.Registry;
import com.extjs.gxt.ui.client.Style.Orientation;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.RpcProxy;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.util.Margins;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Info;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.form.DateField;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.layout.RowData;
import com.extjs.gxt.ui.client.widget.layout.RowLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Pantalla de manteniment de esdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
public class EsdevenimentLayout extends MantenimentLayout<Esdeveniment> {

	/** Servei web per el manteniment de esdeveniments */
	private EsdevenimentServiceAsync eService;

	/** Servei per recuperar les dades de les comarques */
	private ComarcaServiceAsync comarcaService;

	/** Servei per recuperar les dades de les poblacions */
	private PoblacioServiceAsync poblacioService;

	/** Servei per recuperar les dades dels tipus d'esdeveniments */
	private TipusEsdevenimentServiceAsync teService;

	/** Camp id del formulari */
	private NumberField id;

	/** Camp tipus esdeveniment del formulari */
	private ComboBox<TipusEsdeveniment> tipusEsdeveniment;

	/** Camp data inici del formulari */
	private DateField dataInici;

	/** Camp data fi del formulari */
	private DateField dataFi;

	/** Camp adreça del formulari */
	private TextField<String> adreca;

	/** Camp descripció del formulari */
	private TextField<String> descripcio;

	/** Camp comarca del formulari */
	private ComboBox<Comarca> comarca;

	/** Camp poblacio del formulari */
	private ComboBox<Poblacio> poblacio;

	/** Conté la informació de les comarques */
	private ListStore<Comarca> comarques;

	/** Conté la informació de les poblacions */
	private ListStore<Poblacio> poblacions;

	/** Conté la informació dels tipus d'esdeveniments */
	private ListStore<TipusEsdeveniment> tipusEsdeveniments;
	
	/** Instancia del mapa per mostrar els esdeveniments */
	private MapaMantenimentLayout mapaManteniment;
	
	/** Panell que conte el formulari i el mapa */
	ContentPanel formAndMapPanel;

	@Override
	protected void onRender(Element parent, int pos) {
		getGrid().setStateId("esdeveniment");
		setHeaderVisible(false);
		super.onRender(parent, pos);
		// Recuperem els serveis rpc
		eService = (EsdevenimentServiceAsync) Registry.get(ServiceConstants.ESDEVENIMENT_SERVICE);
		comarcaService = (ComarcaServiceAsync) Registry.get(ServiceConstants.COMARCA_SERVICE);
		poblacioService = (PoblacioServiceAsync) Registry.get(ServiceConstants.POBLACIO_SERVICE);
		teService = (TipusEsdevenimentServiceAsync) Registry.get(ServiceConstants.TIPUSESDEVENIMENT_SERVICE);
		// Recuperem la informació per els combobox
		comarcaService.getAll(new AsyncCallback<List<Comarca>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<Comarca> comarcaList) {
				comarques.add(comarcaList);
			}
		});
		poblacioService.getAll(new AsyncCallback<List<Poblacio>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les poblacions", arg0);
			}

			@Override
			public void onSuccess(List<Poblacio> poblacioList) {
				poblacions.add(poblacioList);
			}
		});
		teService.getAll(new AsyncCallback<List<TipusEsdeveniment>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<TipusEsdeveniment> tipusEsdevenimentsList) {
				tipusEsdeveniments.add(tipusEsdevenimentsList);
			}
		});
		getGrid().getSelectionModel().addListener(Events.SelectionChange, new Listener<SelectionChangedEvent<Esdeveniment>>() {
			@Override
			public void handleEvent(SelectionChangedEvent<Esdeveniment> be) {
				if (be.getSelection().size() > 0) {
					tipusEsdeveniment.setValue(be.getSelectedItem().getTipusEsdeveniment());
					comarca.setValue(be.getSelectedItem().getComarca());
					poblacio.setValue(be.getSelectedItem().getPoblacio());
					mapaManteniment.setFeatureId(be.getSelectedItem().getId());
				} else {
					tipusEsdeveniment.clearSelections();
					comarca.clearSelections();
					poblacio.clearSelections();
					mapaManteniment.setFeatureId(0);
				}

			}
		});
		setLayout(new RowLayout());
		setHeight(parent.getClientHeight());
		add(getGridPanel(), new RowData(1, 0.3, new Margins(5, 5, 5, 5)));
		formAndMapPanel = new ContentPanel();
		formAndMapPanel.setHeaderVisible(false);
		formAndMapPanel.setLayout(new RowLayout(Orientation.HORIZONTAL));
		formAndMapPanel.add(getFormPanel(), new RowData(0.4, 1, new Margins(5, 5, 5, 5)));
		this.getFormPanel().setHeading(getMessages().title_esdeveniment_dades());		
		add(formAndMapPanel, new RowData(1, 0.7, new Margins(5, 5, 5, 5)));
		getFormPanel().setLabelWidth(130);
	}

	@Override
	public List<ColumnConfig> createColumnConfig() {
		List<ColumnConfig> columnes = new ArrayList<ColumnConfig>();
		columnes.add(new ColumnConfig("idesdeveniment", getMessages().columna_id(), 50));
		columnes.add(new ColumnConfig("tipusesdevenimentdesc", getMessages().columna_tipusEsdeveniment(), 250));
		ColumnConfig dataIniciColumn = new ColumnConfig("datainici", getMessages().columna_dataInici(), 80);
		dataIniciColumn.setDateTimeFormat(DateTimeFormat.getFormat("dd/MM/yy"));
		columnes.add(dataIniciColumn);
		ColumnConfig dataFiColumn = new ColumnConfig("datafi", getMessages().columna_dataFi(), 80);
		dataFiColumn.setDateTimeFormat(DateTimeFormat.getFormat("dd/MM/yy"));
		columnes.add(dataFiColumn);
		columnes.add(new ColumnConfig("adreca", getMessages().columna_adreca(), 250));
		columnes.add(new ColumnConfig("descripcio", getMessages().columna_descripcio(), 200));
		columnes.add(new ColumnConfig("comarcadesc", getMessages().columna_comarca(), 150));
		columnes.add(new ColumnConfig("poblaciodesc", getMessages().columna_poblacio(), 200));
		return columnes;
	}

	@Override
	public List<Field> createFormFields() {
		List<Field> fields = new ArrayList<Field>();
		// Camp id
		id = new NumberField();
		id.setName("idesdeveniment");
		id.setFieldLabel(getMessages().columna_id());
		id.setAllowDecimals(false);
		id.setReadOnly(true);
		fields.add(id);
		// Camp tipus esdeveniment
		tipusEsdeveniments = new ListStore<TipusEsdeveniment>();
		tipusEsdeveniment = new ComboBox<TipusEsdeveniment>();
		tipusEsdeveniment.setStore(tipusEsdeveniments);
		tipusEsdeveniment.setFieldLabel(getMessages().columna_tipusEsdeveniment());
		tipusEsdeveniment.setValueField("idtipusesdeveniment");
		tipusEsdeveniment.setDisplayField("tipusesdevenimentdesc");		
		tipusEsdeveniment.setTriggerAction(TriggerAction.ALL);
		tipusEsdeveniment.setEditable(false);
		tipusEsdeveniment.setAllowBlank(false);		
		fields.add(tipusEsdeveniment);
		// Camp datainici
		dataInici = new DateField();
		dataInici.setName("datainici");
		dataInici.setFieldLabel(getMessages().columna_dataInici());
		fields.add(dataInici);
		// Camp datafi
		dataFi = new DateField();
		dataFi.setName("datafi");
		dataFi.setFieldLabel(getMessages().columna_dataFi());
		fields.add(dataFi);
		// Camp adreça
		adreca = new TextField<String>();
		adreca.setName("adreca");
		adreca.setFieldLabel(getMessages().columna_adreca());
		fields.add(adreca);
		// Camp descripció
		descripcio = new TextField<String>();
		descripcio.setName("descripcio");
		descripcio.setFieldLabel(getMessages().columna_descripcio());
		descripcio.setAllowBlank(false);
		fields.add(descripcio);
		// Camp comarca
		comarques = new ListStore<Comarca>();
		comarca = new ComboBox<Comarca>();
		comarca.setStore(comarques);;
		comarca.setFieldLabel(getMessages().columna_comarca());
		comarca.setValueField("idcomarca");
		comarca.setDisplayField("comarcadesc");
		comarca.setTriggerAction(TriggerAction.ALL);
		comarca.setEditable(false);
		comarca.setAllowBlank(false);
		fields.add(comarca);
		// Camp poblacio
		poblacions = new ListStore<Poblacio>();
		poblacio = new ComboBox<Poblacio>();
		poblacio.setStore(poblacions);
		poblacio.setFieldLabel(getMessages().columna_poblacio());
		poblacio.setValueField("idpoblacio");
		poblacio.setDisplayField("poblaciodesc");
		poblacio.setTriggerAction(TriggerAction.ALL);
		poblacio.setEditable(false);
		poblacio.setAllowBlank(false);
		fields.add(poblacio);
		return fields;
	}

	@Override
	public RpcProxy<PagingLoadResult<Esdeveniment>> createPaginationProxy() {
		// Creem un proxy de paginació que crida al nostre servei de
		// esdeveniment
		RpcProxy<PagingLoadResult<Esdeveniment>> proxy = new RpcProxy<PagingLoadResult<Esdeveniment>>() {
			@Override
			protected void load(Object loadConfig, AsyncCallback<PagingLoadResult<Esdeveniment>> callback) {
				eService.getAll((PagingLoadConfig) loadConfig, callback);
			}
		};
		return proxy;
	}

	@Override
	public void onSave(FormPanel formPanel) {
		if (formPanel != null) {
			// Recuperem les dades del formulari
			VectorFeature feature = mapaManteniment.getFeature();
			if (feature != null) {
				if (id.getValue() != null) {
					feature.getJSObject().setProperty("state", "Update");
				}
				Attributes attributes = new Attributes();
				attributes.setAttribute("idtipusesdeveniment", tipusEsdeveniment.getSelection().get(0).getId());
				attributes.setAttribute("adreca", adreca.getValue());
				attributes.setAttribute("descripcio", descripcio.getValue());
				attributes.setAttribute("datainici", DateUtils.getGeoserverDate(dataInici.getValue()));
				attributes.setAttribute("datafi", DateUtils.getGeoserverDate(dataFi.getValue()));
				attributes.setAttribute("idcomarca", comarca.getSelection().get(0).getId());
				attributes.setAttribute("idpoblacio", poblacio.getSelection().get(0).getId());
				feature.setAttributes(attributes);
				mapaManteniment.getSaveStrategy().save();
				getFormPanel().clear();
				getGrid().fireEvent(Events.Attach);
				mapaManteniment.setFeatureId(0);
			} else {
				MessageBox.alert(getMessages().dialeg_error(), getMessages().error_localitzacio_obligatoria(), null);
			}
		}
	}

	@Override
	public void onDelete(Esdeveniment deleteObject) {
		if (deleteObject != null) {
			// Intentem esborrar el esdeveniment
			eService.delete(deleteObject, new AsyncCallback<Void>() {
				@Override
				public void onFailure(Throwable arg0) {
					// En cas de error mostrem un missatge generic de error
					MessageBox.alert(getMessages().dialeg_error(), getMessages().esborrar_ko(), null);
				}

				@Override
				public void onSuccess(Void arg0) {
					// En cas de esborrar-se correctament enviem un event al
					// grid perque
					// es recarregui
					getGrid().fireEvent(Events.Attach);
				}
			});
		}
	}
	
	@Override
	public void onCancel() {
		this.getGrid().getSelectionModel().deselectAll();
		this.getFormPanel().clear();
		mapaManteniment.setFeatureId(0);		
		tipusEsdeveniment.clearSelections();
		comarca.clearSelections();
		poblacio.clearSelections();
	}
	
	@Override
	protected void onUnload() {
		onCancel();		
		super.onUnload();
	}
	
	protected void refresh() {
		formAndMapPanel.remove(mapaManteniment);
		mapaManteniment = new MapaMantenimentLayout("esdeveniment", "images/green-dot.png");		
		mapaManteniment.enableCreate();		
		this.mapaManteniment.setHeading(getMessages().title_esdeveniment_localitzacio());
		formAndMapPanel.add(mapaManteniment, new RowData(0.6, 1, new Margins(5, 5, 5, 5)));
		formAndMapPanel.layout();
		super.onAfterLayout();
	}
	
	

}
