package uoc.edu.client.layout;

import java.util.List;
import java.util.Map;

import uoc.edu.client.Messages;
import uoc.edu.shared.model.TipusEsdeveniment;

import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.Style.SelectionMode;
import com.extjs.gxt.ui.client.Style.SortDir;
import com.extjs.gxt.ui.client.binding.FormBinding;
import com.extjs.gxt.ui.client.data.BaseModelData;
import com.extjs.gxt.ui.client.data.BasePagingLoadConfig;
import com.extjs.gxt.ui.client.data.BasePagingLoader;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.PagingLoader;
import com.extjs.gxt.ui.client.data.RpcProxy;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.GridEvent;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.MessageBoxEvent;
import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Dialog;
import com.extjs.gxt.ui.client.widget.Info;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FormButtonBinding;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.FormPanel.LabelAlign;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.grid.ColumnData;
import com.extjs.gxt.ui.client.widget.grid.ColumnModel;
import com.extjs.gxt.ui.client.widget.grid.Grid;
import com.extjs.gxt.ui.client.widget.grid.GridCellRenderer;
import com.extjs.gxt.ui.client.widget.toolbar.PagingToolBar;
import com.google.gwt.core.client.GWT;

/**
 * Creació de un layout genéric que contindrà un grid per mostrar les dades
 * existents i un formulari per editar i crear noves dades.
 * 
 * @author Jordi Farré
 * 
 * @param <T>
 *            Tipus de dada de la que fem manteniment.
 */
public abstract class MantenimentLayout<T extends BaseModelData> extends ContentPanel {

	/** Limit màxim de files mostrades en el grid */
	private int pageLimit = 10;

	/** Missatges i18n */
	private Messages messages;

	/** Grid per mostrar les dades */
	private Grid<T> grid;

	/** Proxy per comunicar el grid amb el servidor */
	private RpcProxy<PagingLoadResult<T>> paginationProxy;

	/** Loader encarregat de gestionar la paginació del grid */
	private PagingLoader<PagingLoadResult<T>> pagingLoader;

	/** Toolbar del grid que conté els botons de paginació */
	private PagingToolBar pagingToolbar;

	/** Dades que es mostren en el grid */
	private ListStore<T> pagingStore;

	/** Configuració de les columnes */
	private List<ColumnConfig> columnConfig;

	/** Model de les columnes */
	private ColumnModel columnModel;

	/** Panell que contindrà el grid */
	private ContentPanel gridPanel;

	/** Llista de camps del formulari */
	private List<Field> formFields;

	/** Panell que contindra el formulari */
	private FormPanel formPanel;

	/** Relaciona el formulari i el grid */
	private FormBinding formBinding;

	/**
	 * Constructor.
	 */
	public MantenimentLayout() {
		init();
	}

	/**
	 * Inicialitza el layout.
	 */
	public void init() {
		createGridPanel();
		createFormPanel();
	}

	/**
	 * Crea el panell del grid.
	 */
	public void createGridPanel() {
		// Inicialitzem la classe per i18n
		messages = GWT.create(Messages.class);
		// Primer creem el panell que contindrá el grid
		gridPanel = new ContentPanel();
		gridPanel.setHeaderVisible(false);
		gridPanel.setBorders(true);
		// Creem el proxy amb el que fem les crides al servidor
		paginationProxy = createPaginationProxy();
		// Creem el loader encarregat de la paginació del grid
		pagingLoader = new BasePagingLoader<PagingLoadResult<T>>(paginationProxy);
		pagingLoader.setRemoteSort(true);
		// Creem el toolbar per la paginació
		pagingToolbar = new PagingToolBar(pageLimit);
		pagingToolbar.bind(pagingLoader);
		// Creem la llista que tindrá el contingut del grid vinculada al loader
		pagingStore = new ListStore<T>(pagingLoader);
		// Creem la configuració de les columnes
		columnConfig = createColumnConfig();
		// Creem una columna addicional amb el botó d'esborrar
		GridCellRenderer<T> deleteColumn = new GridCellRenderer<T>() {
			@Override
			public Object render(T model, String property, ColumnData config, int rowIndex, int colIndex,
					ListStore<T> store, Grid<T> grid) {
				String url = "images/trash.gif";
				return "<img src=\"" + url + "\" />";
			}
		};
		ColumnConfig deleteColumnConfig = new ColumnConfig("delete", 30);
		deleteColumnConfig.setRenderer(deleteColumn);
		deleteColumnConfig.setSortable(false);
		deleteColumnConfig.setFixed(true);
		columnConfig.add(deleteColumnConfig);
		columnModel = new ColumnModel(columnConfig);
		// Creem el grid
		grid = new Grid<T>(pagingStore, columnModel);
		grid.setStateful(true);
		grid.addListener(Events.Attach, new Listener<GridEvent<ModelData>>() {
			public void handleEvent(GridEvent<ModelData> be) {
				PagingLoadConfig config = new BasePagingLoadConfig();
				config.setOffset(0);
				config.setLimit(50);
				Map<String, Object> state = grid.getState();
				if (state.containsKey("offset")) {
					int offset = (Integer) state.get("offset");
					int limit = (Integer) state.get("limit");
					config.setOffset(offset);
					config.setLimit(limit);
				}
				if (state.containsKey("sortField")) {
					config.setSortField((String) state.get("sortField"));
					config.setSortDir(SortDir.valueOf((String) state.get("sortDir")));
				}
				pagingLoader.load(config);
			}
		});
		grid.addListener(Events.CellClick, new Listener<GridEvent<ModelData>>() {
			@Override
			public void handleEvent(GridEvent<ModelData> be) {
				String id = columnModel.getColumnId(be.getColIndex());
				if ("delete".equals(id)) {
					final T deleteObject = pagingStore.getAt(be.getRowIndex());
					MessageBox.confirm(messages.dialeg_confirmacio(), messages.esborrar_confirmacio(),
							new Listener<MessageBoxEvent>() {
								@Override
								public void handleEvent(MessageBoxEvent be) {
									if (Dialog.YES.equals(be.getButtonClicked().getItemId())) {
										onDelete(deleteObject);
									}
								}

							});
				}
			}
		});
		grid.getSelectionModel().setSelectionMode(SelectionMode.SINGLE);
		grid.getSelectionModel().addListener(Events.SelectionChange, new Listener<SelectionChangedEvent<ModelData>>() {
			@Override
			public void handleEvent(SelectionChangedEvent<ModelData> be) {
				if (be.getSelection().size() > 0) {
					formBinding.bind((ModelData) be.getSelection().get(0));
				} else {
					formBinding.unbind();
				}

			}
		});
		grid.setLoadMask(true);
		grid.setBorders(true);
		grid.setAutoHeight(true);
		gridPanel.setBottomComponent(pagingToolbar);
		gridPanel.setScrollMode(Scroll.AUTO);
		gridPanel.add(grid);
		gridPanel.setFrame(true);
	}

	/**
	 * Crea el panell del formulari.
	 */
	public void createFormPanel() {
		formPanel = new FormPanel();
		formPanel.setFrame(true);
		// Creem la llista de camps que tindrá el formulari
		formFields = createFormFields();
		// Recorrem la llista i ho anem afegint al form panel
		for (Field<T> field : formFields) {
			formPanel.add(field);
		}
		// Creem els botons de guardar i cancelar
		Button guardar = new Button(messages.boto_guardar());
		guardar.addSelectionListener(new SelectionListener<ButtonEvent>() {
			@Override
			public void componentSelected(ButtonEvent ce) {
				onSave(formPanel);
			}
		});
		formPanel.addButton(guardar);
		Button cancelar = new Button(messages.boto_cancelar());
		cancelar.addSelectionListener(new SelectionListener<ButtonEvent>() {
			@Override
			public void componentSelected(ButtonEvent ce) {
				formBinding.unbind();
				onCancel();
			}
		});
		// Creem el enllaç entre grid i formulari
		formBinding = new FormBinding(formPanel, true);
		// Creem el enllaç entre el botó de guardar i el formulari
		FormButtonBinding formButtonBinding = new FormButtonBinding(formPanel);
		formButtonBinding.addButton(guardar);

		// Afegim els botons al panel
		formPanel.addButton(cancelar);
		formPanel.setButtonAlign(HorizontalAlignment.LEFT);
	}

	/**
	 * Retorna la configuració de les columnes del grid.
	 * 
	 * @return Columnes del grid.
	 */
	public abstract List<ColumnConfig> createColumnConfig();

	/**
	 * Retorna els camps del formulari.
	 * 
	 * @return Camps del formulari.
	 */
	public abstract List<Field> createFormFields();

	/**
	 * Retorna el proxy encarregat de la comunicació amb el servidor.
	 * 
	 * @return Proxy encarregat de la comunicació amb el servidor.
	 */
	public abstract RpcProxy<PagingLoadResult<T>> createPaginationProxy();

	/**
	 * Métode executat al premer el botó de guardar.
	 * 
	 * @param formPanel
	 *            Formulario con los datos a guardar.
	 */
	public abstract void onSave(FormPanel formPanel);

	/**
	 * Métode executat al premer el botó de esborrar.
	 * 
	 * @param deleteObject
	 *            Objecte a esborrar.
	 */
	public abstract void onDelete(T deleteObject);

	/**
	 * Metode executat al premer el bóto de cancel·lar.
	 */
	public abstract void onCancel();

	/**
	 * @return the messages
	 */
	public Messages getMessages() {
		return messages;
	}

	/**
	 * @param messages
	 *            the messages to set
	 */
	public void setMessages(Messages messages) {
		this.messages = messages;
	}

	/**
	 * @return the grid
	 */
	public Grid<T> getGrid() {
		return grid;
	}

	/**
	 * @param grid
	 *            the grid to set
	 */
	public void setGrid(Grid<T> grid) {
		this.grid = grid;
	}

	/**
	 * @return the pagingLoader
	 */
	public PagingLoader<PagingLoadResult<T>> getPagingLoader() {
		return pagingLoader;
	}

	/**
	 * @param pagingLoader
	 *            the pagingLoader to set
	 */
	public void setPagingLoader(PagingLoader<PagingLoadResult<T>> pagingLoader) {
		this.pagingLoader = pagingLoader;
	}

	/**
	 * @return the gridPanel
	 */
	public ContentPanel getGridPanel() {
		return gridPanel;
	}

	/**
	 * @param gridPanel
	 *            the gridPanel to set
	 */
	public void setGridPanel(ContentPanel gridPanel) {
		this.gridPanel = gridPanel;
	}

	/**
	 * @return the formPanel
	 */
	public FormPanel getFormPanel() {
		return formPanel;
	}

	/**
	 * @param formPanel
	 *            the formPanel to set
	 */
	public void setFormPanel(FormPanel formPanel) {
		this.formPanel = formPanel;
	}

	/**
	 * @return the formBinding
	 */
	public FormBinding getFormBinding() {
		return formBinding;
	}

	/**
	 * @param formBinding
	 *            the formBinding to set
	 */
	public void setFormBinding(FormBinding formBinding) {
		this.formBinding = formBinding;
	}

	/**
	 * @return the pagingToolbar
	 */
	public PagingToolBar getPagingToolbar() {
		return pagingToolbar;
	}

	/**
	 * @param pagingToolbar
	 *            the pagingToolbar to set
	 */
	public void setPagingToolbar(PagingToolBar pagingToolbar) {
		this.pagingToolbar = pagingToolbar;
	}

	/**
	 * @return the pageLimit
	 */
	public int getPageLimit() {
		return pageLimit;
	}

	/**
	 * @param pageLimit
	 *            the pageLimit to set
	 */
	public void setPageLimit(int pageLimit) {
		this.pageLimit = pageLimit;
	}

	/**
	 * @return the pagingStore
	 */
	public ListStore<T> getPagingStore() {
		return pagingStore;
	}

	/**
	 * @param pagingStore
	 *            the pagingStore to set
	 */
	public void setPagingStore(ListStore<T> pagingStore) {
		this.pagingStore = pagingStore;
	}

	/**
	 * @return the columnModel
	 */
	public ColumnModel getColumnModel() {
		return columnModel;
	}

	/**
	 * @param columnModel
	 *            the columnModel to set
	 */
	public void setColumnModel(ColumnModel columnModel) {
		this.columnModel = columnModel;
	}

	/**
	 * @return the columnConfig
	 */
	public List<ColumnConfig> getColumnConfig() {
		return columnConfig;
	}

	/**
	 * @param columnConfig
	 *            the columnConfig to set
	 */
	public void setColumnConfig(List<ColumnConfig> columnConfig) {
		this.columnConfig = columnConfig;
	}

	/**
	 * @return the paginationProxy
	 */
	public RpcProxy<PagingLoadResult<T>> getPaginationProxy() {
		return paginationProxy;
	}

	/**
	 * @param paginationProxy
	 *            the paginationProxy to set
	 */
	public void setPaginationProxy(RpcProxy<PagingLoadResult<T>> paginationProxy) {
		this.paginationProxy = paginationProxy;
	}

	/**
	 * @return the formFields
	 */
	public List<Field> getFormFields() {
		return formFields;
	}

	/**
	 * @param formFields
	 *            the formFields to set
	 */
	public void setFormFields(List<Field> formFields) {
		this.formFields = formFields;
	}

}
