package uoc.edu.client.layout;

import org.gwtopenmaps.openlayers.client.Bounds;
import org.gwtopenmaps.openlayers.client.LonLat;
import org.gwtopenmaps.openlayers.client.MapOptions;
import org.gwtopenmaps.openlayers.client.MapWidget;
import org.gwtopenmaps.openlayers.client.OpenLayers;
import org.gwtopenmaps.openlayers.client.Style;
import org.gwtopenmaps.openlayers.client.control.DrawFeature;
import org.gwtopenmaps.openlayers.client.control.DrawFeature.FeatureAddedListener;
import org.gwtopenmaps.openlayers.client.control.DrawFeatureOptions;
import org.gwtopenmaps.openlayers.client.control.LayerSwitcher;
import org.gwtopenmaps.openlayers.client.control.ModifyFeature;
import org.gwtopenmaps.openlayers.client.control.ModifyFeature.OnModificationListener;
import org.gwtopenmaps.openlayers.client.control.ModifyFeatureOptions;
import org.gwtopenmaps.openlayers.client.control.NavToolbar;
import org.gwtopenmaps.openlayers.client.control.PanZoomBar;
import org.gwtopenmaps.openlayers.client.event.LayerLoadEndListener;
import org.gwtopenmaps.openlayers.client.feature.VectorFeature;
import org.gwtopenmaps.openlayers.client.filter.FeatureIdFilter;
import org.gwtopenmaps.openlayers.client.filter.Filter;
import org.gwtopenmaps.openlayers.client.handler.PointHandler;
import org.gwtopenmaps.openlayers.client.layer.Layer;
import org.gwtopenmaps.openlayers.client.layer.TransitionEffect;
import org.gwtopenmaps.openlayers.client.layer.Vector;
import org.gwtopenmaps.openlayers.client.layer.VectorOptions;
import org.gwtopenmaps.openlayers.client.layer.WMS;
import org.gwtopenmaps.openlayers.client.layer.WMSOptions;
import org.gwtopenmaps.openlayers.client.layer.WMSParams;
import org.gwtopenmaps.openlayers.client.protocol.WFSProtocol;
import org.gwtopenmaps.openlayers.client.protocol.WFSProtocolOptions;
import org.gwtopenmaps.openlayers.client.strategy.Strategy;

import uoc.edu.client.openlayers.FixedStrategy;
import uoc.edu.client.openlayers.OSMExtended;
import uoc.edu.client.openlayers.RefreshStrategy;
import uoc.edu.client.openlayers.SaveStrategy;

import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Info;
import com.google.gwt.user.client.Element;

/**
 * Layout per afegir un mapa amb wfs-t en qualsevol pantalla de manteniment.
 * 
 * @author Jordi Farré
 * 
 */
public class MapaMantenimentLayout extends ContentPanel {
	
	

	/** Identificador del registre seleccionat en aquest moment */
	private Integer featureId = 0;
	
	/** Icono que va a utilizar para señalar la localizacion */
	private String icon;

	/** Nom del layer al que accedirem per wfs */
	private String wfsLayerName;

	/** Widget per mostrar el mapa */
	private MapWidget mapWidget;

	/** Opcions per el protocol wfs */
	private WFSProtocolOptions wfsProtocolOptions;
	
	/** Capa base de tipus osm */
	private OSMExtended osm;

	/** Layer wms */
	private WMS wmsLayer;

	/** Layer wfs */
	private Vector wfsLayer;

	/** Estrategy per guardar amb wfs-t */
	private SaveStrategy saveStrategy;

	/** Estrategy per refrescar el layer wfs-t */
	private RefreshStrategy refreshStrategy;

	/** Control per pintar un punt en el layer wfs */
	private DrawFeature drawFeature;

	/** Control per moure un punt en el layer wfs */
	private ModifyFeature modifyFeature;

	/** Feature que tenim actualment en el mapa */
	private VectorFeature feature;
	
	/** Control per moure el mapa i per fer zoom */
	private NavToolbar navToolbar;

	/**
	 * Constructor.
	 * 
	 * @param wfsLayerName
	 *            Nom de la capa wfs.
	 */
	public MapaMantenimentLayout(String wfsLayerName, String icon) {
		this.wfsLayerName = wfsLayerName;
		this.icon = icon;
		this.setId(wfsLayerName);
		// Creem el mapa i les diferents capes
		createMap();
		createBaseLayer();
		createWfsLayer();
		// Afegim els controls necessaris i seleccionem un centre per el mapa
		navToolbar = new NavToolbar();
		mapWidget.getMap().addControl(new PanZoomBar());	
		mapWidget.getMap().addLayers(new Layer[] {osm, wmsLayer, wfsLayer });
		mapWidget.getMap().setCenter(new LonLat(238408, 5069840), 12);
		// Creem el control que ens permetrà afegir un nou punt al mapa
		DrawFeatureOptions drawFeatureOptions = new DrawFeatureOptions();
		drawFeatureOptions.onFeatureAdded(new FeatureAddedListener() {
			@Override
			public void onFeatureAdded(VectorFeature vectorFeature) {
				// Actualitzem la variable amb la feature afegida
				enableModify();
				feature = vectorFeature;
			}
		});
		drawFeature = new DrawFeature(wfsLayer, new PointHandler(), drawFeatureOptions);
		drawFeature.deactivate();		
		mapWidget.getMap().addControl(drawFeature);		
		// Creem el control que ens permetra modificar una feature
		ModifyFeatureOptions modifyFeatureOptions = new ModifyFeatureOptions();
		modifyFeatureOptions.onModification(new OnModificationListener() {
			@Override
			public void onModification(VectorFeature vectorFeature) {
				// Modifiquem la variable amb la feature modificada
				feature = vectorFeature;
			}
		});
		modifyFeature = new ModifyFeature(wfsLayer, modifyFeatureOptions);
		modifyFeature.deactivate();
		mapWidget.getMap().addControl(modifyFeature);

	}

	@Override
	protected void onRender(Element parent, int pos) {
		super.onRender(parent, pos);
		this.add(mapWidget);
	}

	/**
	 * Crea el mapa.
	 */
	private void createMap() {
		// Definim les opcions per defecte del mapa
		Bounds bounds = new Bounds(12746, 4932506, 374628, 5295759);
		MapOptions defaultMapOptions = new MapOptions();
		defaultMapOptions.removeDefaultControls();
		defaultMapOptions.setProjection("EPSG:900913");
		defaultMapOptions.setMaxExtent(bounds);
		defaultMapOptions.setMaxResolutionToAuto();
		// Instanciem el widget
		mapWidget = new MapWidget("100%", "100%", defaultMapOptions);
	}

	/**
	 * Crea les capes base.
	 */
	private void createBaseLayer() {
		// Definim la capa osm
		osm = new OSMExtended("osm");
		// Definim els parametres del wms
		WMSParams wmsParams = new WMSParams();
		wmsParams.setFormat("image/png");
		wmsParams.setLayers("xcs:planet_osm_line");
		wmsParams.setIsTransparent(true);
		wmsParams.setStyles("");
		// Definim les opcions del layer
		WMSOptions wmsLayerParams = new WMSOptions();
		wmsLayerParams.setUntiled();
		wmsLayerParams.setLayerOpacity(0.3);
		wmsLayerParams.setTransitionEffect(TransitionEffect.RESIZE);
		// Definim el layer que crida al wms de geoserver
		wmsLayer = new WMS("base_" + wfsLayerName, "http://localhost:8080/geoserver/wms", wmsParams, wmsLayerParams);
	}

	/**
	 * Crea la capa wfs-t.
	 */
	private void createWfsLayer() {
		// Es necessari un proxy per el correcte funcionament de wfs-t.
		// GWT-Openlayers ja
		// te una implementació d'aquest proxy
		OpenLayers.setProxyHost("gwtOpenLayersProxy?targetURL=");
		// Creem les opcions de la capa wfs
		wfsProtocolOptions = new WFSProtocolOptions();
		wfsProtocolOptions.setUrl("http://localhost:8080/geoserver/wfs");
		wfsProtocolOptions.setVersion("1.1.0");
		wfsProtocolOptions.setFeatureType(wfsLayerName);
		wfsProtocolOptions.setSrsName("EPSG:900913");
		// Afegim el filtre que inicialment filtrara amb una feature que no
		// existeix,
		// per tal de que no surti cap punt en el mapa
		Filter filter = new FeatureIdFilter(new String[] { wfsLayerName + "." + featureId });
		wfsProtocolOptions.setFilter(filter);
		wfsProtocolOptions.setFeatureNameSpace("http://uoc.edu/xcs");
		wfsProtocolOptions.setGeometryName("localitzacio");

		WFSProtocol wfsProtocol = new WFSProtocol(wfsProtocolOptions);

		VectorOptions vectorOptions = new VectorOptions();
		vectorOptions.setProtocol(wfsProtocol);
		saveStrategy = new SaveStrategy();
		refreshStrategy = new RefreshStrategy();
		// En les opcions de wfs, indiquem:
		// FixedStrategy: Necessari per carregar els punts en el mapa.
		// SaveStrategy: Necessari per fer commit de la transaccio wfs.
		// RefreshStrategy: Necessari per refrescar el layer wfs.
		vectorOptions.setStrategies(new Strategy[] { new FixedStrategy(), saveStrategy, refreshStrategy });
		Style style = new Style();
		style.setExternalGraphic(icon);
		style.setGraphicSize(30, 30);
		style.setFillOpacity(1);
		vectorOptions.setStyle(style);
		wfsLayer = new Vector(wfsLayerName, vectorOptions);		
		wfsLayer.addLayerLoadEndListener(new LayerLoadEndListener() {
			@Override
			public void onLoadEnd(LoadEndEvent eventObject) {
				if (wfsLayer.getNumberOfFeatures() > 0) {
					// Si al carregar el layer wfs ja tenim una feature
					// la informem en la variable i posem el centre en la
					// localitzacio d'aquesta feature
					feature = wfsLayer.getFeatures()[0];
					mapWidget.getMap().setCenter(feature.getCenterLonLat());
				} else {
					feature = null;
				}
			}
		});
	}

	/**
	 * @param id
	 *            the id to set
	 */
	public void setFeatureId(Integer featureId) {
		// Al informar el id de la feature, aplicarem el filtre per aquesta
		// feature i refrescarem el layer wfs.		
		this.featureId = featureId;
		if (featureId > 0) {
			enableModify();
		} else {
			enableCreate();
		}
		Filter filter = new FeatureIdFilter(new String[] { wfsLayerName + "." + featureId });
		wfsProtocolOptions.setFilter(filter);
		refreshStrategy.refresh();
	}

	/**
	 * Activa el draw feature, per poder seleccionar un nou punt.
	 */
	public void enableCreate() {
		modifyFeature.deactivate();
		drawFeature.activate();
		mapWidget.getMap().removeControl(navToolbar);
	}

	/**
	 * Activa el modify feature, per poder mour un punt de lloc.
	 */
	public void enableModify() {
		drawFeature.deactivate();
		modifyFeature.activate();
		mapWidget.getMap().addControl(navToolbar);	
	}

	/**
	 * @return the mapWidget
	 */
	public MapWidget getMapWidget() {
		return mapWidget;
	}

	/**
	 * @param mapWidget
	 *            the mapWidget to set
	 */
	public void setMapWidget(MapWidget mapWidget) {
		this.mapWidget = mapWidget;
	}

	/**
	 * @return the wfsLayerName
	 */
	public String getWfsLayerName() {
		return wfsLayerName;
	}

	/**
	 * @param wfsLayerName
	 *            the wfsLayerName to set
	 */
	public void setWfsLayerName(String wfsLayerName) {
		this.wfsLayerName = wfsLayerName;
	}

	/**
	 * @return the wfsProtocolOptions
	 */
	public WFSProtocolOptions getWfsProtocolOptions() {
		return wfsProtocolOptions;
	}

	/**
	 * @param wfsProtocolOptions
	 *            the wfsProtocolOptions to set
	 */
	public void setWfsProtocolOptions(WFSProtocolOptions wfsProtocolOptions) {
		this.wfsProtocolOptions = wfsProtocolOptions;
	}

	/**
	 * @return the wmsLayer
	 */
	public WMS getWmsLayer() {
		return wmsLayer;
	}

	/**
	 * @param wmsLayer
	 *            the wmsLayer to set
	 */
	public void setWmsLayer(WMS wmsLayer) {
		this.wmsLayer = wmsLayer;
	}

	/**
	 * @return the wfsLayer
	 */
	public Vector getWfsLayer() {
		return wfsLayer;
	}

	/**
	 * @param wfsLayer
	 *            the wfsLayer to set
	 */
	public void setWfsLayer(Vector wfsLayer) {
		this.wfsLayer = wfsLayer;
	}

	/**
	 * @return the saveStrategy
	 */
	public SaveStrategy getSaveStrategy() {
		return saveStrategy;
	}

	/**
	 * @param saveStrategy
	 *            the saveStrategy to set
	 */
	public void setSaveStrategy(SaveStrategy saveStrategy) {
		this.saveStrategy = saveStrategy;
	}

	/**
	 * @return the refreshStrategy
	 */
	public RefreshStrategy getRefreshStrategy() {
		return refreshStrategy;
	}

	/**
	 * @param refreshStrategy
	 *            the refreshStrategy to set
	 */
	public void setRefreshStrategy(RefreshStrategy refreshStrategy) {
		this.refreshStrategy = refreshStrategy;
	}

	/**
	 * @return the drawFeature
	 */
	public DrawFeature getDrawFeature() {
		return drawFeature;
	}

	/**
	 * @param drawFeature
	 *            the drawFeature to set
	 */
	public void setDrawFeature(DrawFeature drawFeature) {
		this.drawFeature = drawFeature;
	}

	/**
	 * @return the modifyFeature
	 */
	public ModifyFeature getModifyFeature() {
		return modifyFeature;
	}

	/**
	 * @param modifyFeature
	 *            the modifyFeature to set
	 */
	public void setModifyFeature(ModifyFeature modifyFeature) {
		this.modifyFeature = modifyFeature;
	}

	/**
	 * @return the featureId
	 */
	public Integer getFeatureId() {
		return featureId;
	}

	/**
	 * @return the feature
	 */
	public VectorFeature getFeature() {
		return feature;
	}

	/**
	 * @param feature
	 *            the feature to set
	 */
	public void setFeature(VectorFeature feature) {
		this.feature = feature;
	}

}
