package uoc.edu.client.layout;

import uoc.edu.client.Messages;

import com.extjs.gxt.ui.client.event.MenuEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.Viewport;
import com.extjs.gxt.ui.client.widget.layout.FillLayout;
import com.extjs.gxt.ui.client.widget.menu.Menu;
import com.extjs.gxt.ui.client.widget.menu.MenuBar;
import com.extjs.gxt.ui.client.widget.menu.MenuBarItem;
import com.extjs.gxt.ui.client.widget.menu.MenuItem;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;

/**
 * Crea la estructura principal de la pagina i el menu.
 * 
 * @author Jordi Farré
 * 
 */
public class PrincipalLayout extends ContentPanel {
	public PrincipalLayout() {
	}
	
	/** Constant per la entrada de menú de buscar */
	public static final String SEARCH = "SEARCH";
	
	/** Constant per la entrada de menú de xcs */
	public static final String XCS = "XCS";
	
	/** Constant per la entrada de menú de productor */
	public static final String PRODUCTOR = "PRODUCTOR";
	
	/** Constant per la entrada de menú de esdeveniment */
	public static final String ESDEVENIMENT = "ESDEVENIMENT";

	/** Constant per la entrada de menú de tipus de esdeveniment */
	public static final String TIPUS_ESDEVENIMENT = "TIPUSESDEVENIMENT";

	/** Constant per la entrada de menú de tipus de productor */
	public static final String TIPUS_PRODUCTOR = "TIPUSPRODUCTOR";
	
	/** Constant per la entrada de menú de producte */
	public static final String PRODUCTE = "PRODUCTE";

	/** Estructura principal de la pàgina */
	private final FillLayout principal = new FillLayout();

	/** Panell que contindrà el contingut */
	private final ContentPanel contingut = new ContentPanel();

	/** Referencia per i18n */
	private Messages messages = GWT.create(Messages.class);
	
	/** Pantalla de cerca de xcs, productors i esdeveniments */
	SearchLayout searchLayout = new SearchLayout();
	
	/** Pantalla de manteniment de xcs */ 
	XcsLayout xcsLayout = new XcsLayout();
	
	/** Pantalla de manteniment de productors */
	ProductorLayout pLayout = new ProductorLayout();
	
	/** Pantalla de manteniment de esdeveniments */
	EsdevenimentLayout eLayout = new EsdevenimentLayout();

	/** Pantalla de manteniment de tipus de esdeveniments */
	TipusEsdevenimentLayout teLayout = new TipusEsdevenimentLayout();

	/** Pantalla de manteniment de tipus de productors */
	TipusProductorLayout tpLayout = new TipusProductorLayout();
	
	/** Pantalla de manteniment de productes */
	ProducteLayout producteLayout = new ProducteLayout();

	/**
	 * Inicialitzem la estructura principal.
	 */
	@Override
	protected void onRender(Element parent, int index) {
		setHeaderVisible(false);
		super.onRender(parent, index);
		setLayout(principal);		
		setStyleAttribute("padding", "10px");
		// Creem el menu
		createMenu();
		// Inicialment mostrem la pantalla de buscar
		contingut.add(searchLayout);
		contingut.setHeading(messages.title_cerca());
		add(contingut);
	}

	/**
	 * Creació del menu principal.
	 */
	private void createMenu() {
		// Instanciem el listener per el menu
		MenuSelectionListener menuSelectionListener = new MenuSelectionListener();
		// Creem la barra de menu
		MenuBar menuBar = new MenuBar();
		// Creem els menus corresponents a buscar
		Menu subBuscar = new Menu();
		MenuItem buscarItem = new MenuItem(messages.menu_buscar());
		buscarItem.setId(SEARCH);
		buscarItem.addSelectionListener(menuSelectionListener);
		subBuscar.add(buscarItem);
		// Creem els menus corresponents a manteniment
		Menu subManteniment = new Menu();
		MenuItem xcsItem = new MenuItem(messages.menu_xcs());
		xcsItem.setId(XCS);
		xcsItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(xcsItem);
		MenuItem productorItem = new MenuItem(messages.menu_productor());
		productorItem.setId(PRODUCTOR);
		productorItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(productorItem);
		MenuItem esdevenimentItem = new MenuItem(messages.menu_esdeveniment());
		esdevenimentItem.setId(ESDEVENIMENT);
		esdevenimentItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(esdevenimentItem);
		MenuItem producteItem = new MenuItem(messages.menu_producte());
		producteItem.setId(PRODUCTE);
		producteItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(producteItem);
		MenuItem tipusEsdevenimentItem = new MenuItem(messages.menu_tipusEsdeveniment());
		tipusEsdevenimentItem.setId(TIPUS_ESDEVENIMENT);
		tipusEsdevenimentItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(tipusEsdevenimentItem);
		MenuItem tipusProductorItem = new MenuItem(messages.menu_tipusProductor());
		tipusProductorItem.setId(TIPUS_PRODUCTOR);
		tipusProductorItem.addSelectionListener(menuSelectionListener);
		subManteniment.add(tipusProductorItem);
		// Afegim els menus a la barra de menus
		menuBar.add(new MenuBarItem(messages.menu_buscar(), subBuscar));
		menuBar.add(new MenuBarItem(messages.menu_manteniment(), subManteniment));
		// Afegim la barra de menú a la finestra
		contingut.setTopComponent(menuBar);
	}	

	/**
	 * Controla la selecció del menu.
	 * 
	 * @author Jordi Farré
	 * 
	 */
	private class MenuSelectionListener extends SelectionListener<MenuEvent> {

		@Override
		public void componentSelected(MenuEvent ce) {
			MenuItem source = (MenuItem) ce.getItem();
			contingut.removeAll();
			if (TIPUS_ESDEVENIMENT.equals(source.getId())) {				
				contingut.add(teLayout);
				contingut.setHeading(messages.title_manteniment_tipusEsdeveniment());
				contingut.layout();
			} else if (TIPUS_PRODUCTOR.equals(source.getId())) {
				contingut.add(tpLayout);
				contingut.setHeading(messages.title_manteniment_tipusProductor());
				contingut.layout();
			} else if (ESDEVENIMENT.equals(source.getId())) {
				contingut.add(eLayout);
				contingut.setHeading(messages.title_manteniment_esdeveniment());
				contingut.layout();
				eLayout.refresh();
			} else if (PRODUCTOR.equals(source.getId())) {
				contingut.add(pLayout);
				contingut.setHeading(messages.title_manteniment_productor());
				contingut.layout();
				pLayout.refresh();				
			} else if (XCS.equals(source.getId())) {
				contingut.add(xcsLayout);
				contingut.setHeading(messages.title_manteniment_xcs());
				contingut.layout();
				xcsLayout.refresh();
			} else if (SEARCH.equals(source.getId())) {
				contingut.add(searchLayout);
				contingut.setHeading(messages.title_cerca());
				contingut.layout();
			} else if (PRODUCTE.equals(source.getId())) {
				contingut.add(producteLayout);
				contingut.setHeading(messages.title_manteniment_producte());
				contingut.layout();
			}			
		}

	}

}
