package uoc.edu.client.layout;

import java.util.ArrayList;
import java.util.List;

import org.gwtopenmaps.openlayers.client.feature.VectorFeature;
import org.gwtopenmaps.openlayers.client.util.Attributes;

import uoc.edu.client.service.ComarcaServiceAsync;
import uoc.edu.client.service.PoblacioServiceAsync;
import uoc.edu.client.service.ProductorServiceAsync;
import uoc.edu.client.service.ServiceConstants;
import uoc.edu.client.service.TipusProductorServiceAsync;
import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.Producte;
import uoc.edu.shared.model.Productor;
import uoc.edu.shared.model.TipusProductor;

import com.extjs.gxt.ui.client.Registry;
import com.extjs.gxt.ui.client.Style.Orientation;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.RpcProxy;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.util.Margins;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Info;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.form.DualListField;
import com.extjs.gxt.ui.client.widget.form.DualListField.Mode;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.layout.RowData;
import com.extjs.gxt.ui.client.widget.layout.RowLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Pantalla de manteniment de productors.
 * 
 * @author Jordi Farré
 * 
 */
public class ProductorLayout extends MantenimentLayout<Productor> {

	/** Servei web per el manteniment de productors */
	private ProductorServiceAsync pService;

	/** Servei per recuperar les dades de les comarques */
	private ComarcaServiceAsync comarcaService;

	/** Servei per recuperar les dades de les poblacions */
	private PoblacioServiceAsync poblacioService;

	/** Servei per recuperar les dades dels tipus de productors */
	private TipusProductorServiceAsync tpService;

	/** Camp id del formulari */
	private NumberField id;

	/** Camp tipus productor del formulari */
	private ComboBox<TipusProductor> tipusProductor;

	/** Camp adreça del formulari */
	private TextField<String> adreca;

	/** Camp descripció del formulari */
	private TextField<String> descripcio;

	/** Camp teléfon del formulari */
	private TextField<String> telefon;

	/** Camp email del formulari */
	private TextField<String> email;

	/** Camp comarca del formulari */
	private ComboBox<Comarca> comarca;

	/** Camp poblacio del formulari */
	private ComboBox<Poblacio> poblacio;

	/** Selecció d'esdeveniments pel formulari */
	private DualListField<Esdeveniment> esdeveniments;

	/** Selecció productes pel formulari */
	private DualListField<Producte> productes;

	/** Conté la informació de les comarques */
	private ListStore<Comarca> comarques;

	/** Conté la informació de les poblacions */
	private ListStore<Poblacio> poblacions;

	/** Conté la informació dels tipus de productors */
	private ListStore<TipusProductor> tipusProductors;

	/** Esdeveniments disponibles */
	private ListStore<Esdeveniment> esdevenimentsSource;

	/** Esdeveniments seleccionats */
	private ListStore<Esdeveniment> esdevenimentsTarget;

	/** Productes disponibles */
	private ListStore<Producte> productesSource;

	/** Productes seleccionats */
	private ListStore<Producte> productesTarget;

	/** Instancia del mapa per mostrar els productors */
	private MapaMantenimentLayout mapaManteniment;
	
	/** Panell amb el formulari i el mapa */
	ContentPanel formAndMapPanel;

	@Override
	protected void onRender(Element parent, int pos) {
		getGrid().setStateId("productor");
		setHeaderVisible(false);
		super.onRender(parent, pos);
		// Recuperem els serveis rpc
		pService = (ProductorServiceAsync) Registry.get(ServiceConstants.PRODUCTOR_SERVICE);
		comarcaService = (ComarcaServiceAsync) Registry.get(ServiceConstants.COMARCA_SERVICE);
		poblacioService = (PoblacioServiceAsync) Registry.get(ServiceConstants.POBLACIO_SERVICE);
		tpService = (TipusProductorServiceAsync) Registry.get(ServiceConstants.TIPUSPRODUCTOR_SERVICE);
		// Recuperem la informació per els combobox
		comarcaService.getAll(new AsyncCallback<List<Comarca>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<Comarca> comarcaList) {
				comarques.add(comarcaList);
			}
		});
		poblacioService.getAll(new AsyncCallback<List<Poblacio>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les poblacions", arg0);
			}

			@Override
			public void onSuccess(List<Poblacio> poblacioList) {
				poblacions.add(poblacioList);
			}
		});
		tpService.getAll(new AsyncCallback<List<TipusProductor>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<TipusProductor> tipusProductorsList) {
				tipusProductors.add(tipusProductorsList);
			}
		});
		loadEsdevenimentsSource(null);
		loadEsdevenimentsTarget(null);
		loadProductesSource(null);
		loadProductesTarget(null);
		getGrid().getSelectionModel().addListener(Events.SelectionChange,
				new Listener<SelectionChangedEvent<Productor>>() {
					@Override
					public void handleEvent(SelectionChangedEvent<Productor> be) {
						if (be.getSelection().size() > 0) {
							loadEsdevenimentsSource(be.getSelectedItem().getId());
							loadEsdevenimentsTarget(be.getSelectedItem().getId());
							loadProductesSource(be.getSelectedItem().getId());
							loadProductesTarget(be.getSelectedItem().getId());
							tipusProductor.setValue(be.getSelectedItem().getTipusProductor());
							comarca.setValue(be.getSelectedItem().getComarca());
							poblacio.setValue(be.getSelectedItem().getPoblacio());
							esdeveniments.setEnabled(true);
							productes.setEnabled(true);
							mapaManteniment.setFeatureId(be.getSelectedItem().getId());
						} else {
							loadEsdevenimentsSource(null);
							loadEsdevenimentsTarget(null);
							loadEsdevenimentsSource(null);
							loadEsdevenimentsTarget(null);
							tipusProductor.clearSelections();
							comarca.clearSelections();
							poblacio.clearSelections();
							esdeveniments.setEnabled(false);
							productes.setEnabled(false);
							mapaManteniment.setFeatureId(0);
						}

					}
				});
		setLayout(new RowLayout());
		setHeight(parent.getClientHeight());
		add(getGridPanel(), new RowData(1, 0.3, new Margins(5, 5, 5, 5)));

		formAndMapPanel = new ContentPanel();
		formAndMapPanel.setHeaderVisible(false);
		formAndMapPanel.setLayout(new RowLayout(Orientation.HORIZONTAL));
		FormPanel formPanel = getFormPanel();
		formPanel.setLabelWidth(110);
		formPanel.setScrollMode(Scroll.AUTOY);
		this.getFormPanel().setHeading(getMessages().title_productor_dades());
		formAndMapPanel.add(formPanel, new RowData(0.4, 1, new Margins(5, 5, 5, 5)));
		add(formAndMapPanel, new RowData(1, 0.7, new Margins(5, 5, 5, 5)));
	}

	@Override
	public List<ColumnConfig> createColumnConfig() {
		List<ColumnConfig> columnes = new ArrayList<ColumnConfig>();
		columnes.add(new ColumnConfig("idproductor", getMessages().columna_id(), 50));
		columnes.add(new ColumnConfig("tipusproductordesc", getMessages().columna_tipusProductor(), 250));
		columnes.add(new ColumnConfig("adreca", getMessages().columna_adreca(), 250));
		columnes.add(new ColumnConfig("descripcio", getMessages().columna_descripcio(), 200));
		columnes.add(new ColumnConfig("telefon", getMessages().columna_descripcio(), 100));
		columnes.add(new ColumnConfig("email", getMessages().columna_descripcio(), 100));
		columnes.add(new ColumnConfig("comarcadesc", getMessages().columna_comarca(), 150));
		columnes.add(new ColumnConfig("poblaciodesc", getMessages().columna_poblacio(), 200));
		return columnes;
	}

	@Override
	public List<Field> createFormFields() {
		List<Field> fields = new ArrayList<Field>();
		// Camp id
		id = new NumberField();
		id.setName("idproductor");
		id.setFieldLabel(getMessages().columna_id());
		id.setAllowDecimals(false);
		id.setReadOnly(true);
		fields.add(id);
		// Camp tipus productor
		tipusProductors = new ListStore<TipusProductor>();
		tipusProductor = new ComboBox<TipusProductor>();
		tipusProductor.setStore(tipusProductors);
		tipusProductor.setFieldLabel(getMessages().columna_tipusProductor());
		tipusProductor.setValueField("idtipusproductor");
		tipusProductor.setDisplayField("tipusproductordesc");
		tipusProductor.setEditable(false);
		tipusProductor.setAllowBlank(false);
		tipusProductor.setTriggerAction(TriggerAction.ALL);
		fields.add(tipusProductor);
		// Camp adreça
		adreca = new TextField<String>();
		adreca.setName("adreca");
		adreca.setFieldLabel(getMessages().columna_adreca());
		fields.add(adreca);
		// Camp descripció
		descripcio = new TextField<String>();
		descripcio.setName("descripcio");
		descripcio.setFieldLabel(getMessages().columna_descripcio());
		descripcio.setAllowBlank(false);
		fields.add(descripcio);
		// Camp telefon
		telefon = new TextField<String>();
		telefon.setName("telefon");
		telefon.setFieldLabel(getMessages().columna_telefon());
		fields.add(telefon);
		// Camp descripció
		email = new TextField<String>();
		email.setName("email");
		email.setFieldLabel(getMessages().columna_email());
		fields.add(email);
		// Camp comarca
		comarques = new ListStore<Comarca>();
		comarca = new ComboBox<Comarca>();
		comarca.setStore(comarques);
		comarca.setFieldLabel(getMessages().columna_comarca());
		comarca.setValueField("idcomarca");
		comarca.setDisplayField("comarcadesc");
		comarca.setEditable(false);
		comarca.setAllowBlank(false);
		comarca.setTriggerAction(TriggerAction.ALL);
		fields.add(comarca);
		// Camp poblacio
		poblacions = new ListStore<Poblacio>();
		poblacio = new ComboBox<Poblacio>();
		poblacio.setStore(poblacions);
		poblacio.setFieldLabel(getMessages().columna_poblacio());
		poblacio.setValueField("idpoblacio");
		poblacio.setDisplayField("poblaciodesc");
		poblacio.setEditable(false);
		poblacio.setAllowBlank(false);
		poblacio.setTriggerAction(TriggerAction.ALL);
		fields.add(poblacio);
		// Control per seleccionar els esdeveniments
		esdeveniments = new DualListField<Esdeveniment>();
		esdeveniments.setMode(Mode.INSERT);
		esdeveniments.setFieldLabel(getMessages().columna_esdeveniments());
		esdevenimentsSource = new ListStore<Esdeveniment>();
		esdevenimentsTarget = new ListStore<Esdeveniment>();
		esdeveniments.setEnabled(false);
		ListField<Esdeveniment> source = esdeveniments.getFromList();
		source.setStore(esdevenimentsSource);
		source.setDisplayField("descripcio");
		ListField<Esdeveniment> target = esdeveniments.getToList();
		target.setStore(esdevenimentsTarget);
		target.setDisplayField("descripcio");
		fields.add(esdeveniments);
		// Control per seleccionar els productes
		productes = new DualListField<Producte>();
		productes.setMode(Mode.INSERT);
		productes.setFieldLabel(getMessages().columna_productes());
		productesSource = new ListStore<Producte>();
		productesTarget = new ListStore<Producte>();
		productes.setEnabled(false);
		ListField<Producte> source1 = productes.getFromList();
		source1.setStore(productesSource);
		source1.setDisplayField("productedesc");
		ListField<Producte> target1 = productes.getToList();
		target1.setStore(productesTarget);
		target1.setDisplayField("productedesc");
		fields.add(productes);
		return fields;
	}

	@Override
	public RpcProxy<PagingLoadResult<Productor>> createPaginationProxy() {
		// Creem un proxy de paginació que crida al nostre servei de
		// productor
		RpcProxy<PagingLoadResult<Productor>> proxy = new RpcProxy<PagingLoadResult<Productor>>() {
			@Override
			protected void load(Object loadConfig, AsyncCallback<PagingLoadResult<Productor>> callback) {
				pService.getAll((PagingLoadConfig) loadConfig, callback);
			}
		};
		return proxy;
	}

	@Override
	public void onSave(FormPanel formPanel) {
		if (formPanel != null) {
			// Recuperem les dades del formulari
			VectorFeature feature = mapaManteniment.getFeature();
			if (feature != null) {
				if (id.getValue() != null) {
					feature.getJSObject().setProperty("state", "Update");
				}
				Attributes attributes = new Attributes();
				attributes.setAttribute("idtipusproductor", tipusProductor.getValue().getId());
				attributes.setAttribute("adreca", adreca.getValue());
				attributes.setAttribute("descripcio", descripcio.getValue());
				attributes.setAttribute("telefon", telefon.getValue());
				attributes.setAttribute("email", email.getValue());
				attributes.setAttribute("idcomarca", comarca.getValue().getId());
				attributes.setAttribute("idpoblacio", poblacio.getValue().getId());
				feature.setAttributes(attributes);
				mapaManteniment.getSaveStrategy().save();				
				// Guardem els esdeveniments
				if (id.getValue() != null) {
					int idInt = id.getValue().intValue();
					pService.saveEsdeveniments(esdeveniments.getFromList().getStore().getModels(), esdeveniments
							.getToList().getStore().getModels(), idInt, new AsyncCallback<Void>() {
						@Override
						public void onFailure(Throwable arg0) {
							// En cas de error mostrem un missatge generic
							MessageBox.alert(getMessages().dialeg_error(), getMessages().guardar_ko(), null);
							getFormPanel().clear();
						}

						@Override
						public void onSuccess(Void arg0) {							
						}
					});
					pService.saveProductes(productes.getFromList().getStore().getModels(), productes
							.getToList().getStore().getModels(), idInt, new AsyncCallback<Void>() {
						@Override
						public void onFailure(Throwable arg0) {
							// En cas de error mostrem un missatge generic
							MessageBox.alert(getMessages().dialeg_error(), getMessages().guardar_ko(), null);
							getFormPanel().clear();
						}

						@Override
						public void onSuccess(Void arg0) {
						}
					});					
				} 
				getGrid().fireEvent(Events.Attach);
				mapaManteniment.setFeatureId(0);
				getFormPanel().clear();
			} else {
				MessageBox.alert(getMessages().dialeg_error(), getMessages().error_localitzacio_obligatoria(), null);
			}			
		}
	}

	@Override
	public void onDelete(Productor deleteObject) {
		if (deleteObject != null) {
			// Intentem esborrar el esdeveniment
			pService.delete(deleteObject, new AsyncCallback<Void>() {
				@Override
				public void onFailure(Throwable arg0) {
					// En cas de error mostrem un missatge generic de error
					MessageBox.alert(getMessages().dialeg_error(), getMessages().esborrar_ko(), null);
				}

				@Override
				public void onSuccess(Void arg0) {
					// En cas de esborrar-se correctament enviem un event al
					// grid perque
					// es recarregui
					getGrid().fireEvent(Events.Attach);
				}
			});
		}
	}

	/**
	 * Carrega el selector de esdeveniments disponibles.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 */
	public void loadEsdevenimentsSource(Integer idProductor) {
		pService.getAllEsdevenimentsSource(idProductor, new AsyncCallback<List<Esdeveniment>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els esdeveniments disponibles", arg0);
			}

			@Override
			public void onSuccess(List<Esdeveniment> esdeveniments) {
				esdevenimentsSource.removeAll();
				esdevenimentsSource.add(esdeveniments);
			}
		});
	}

	/**
	 * Carrega el selector de esdeveniments associats.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 */
	public void loadEsdevenimentsTarget(Integer idProductor) {
		pService.getAllEsdevenimentsTarget(idProductor, new AsyncCallback<List<Esdeveniment>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els esdeveniments seleccionats", arg0);
			}

			@Override
			public void onSuccess(List<Esdeveniment> esdeveniments) {
				esdevenimentsTarget.removeAll();
				esdevenimentsTarget.add(esdeveniments);
			}
		});
	}

	/**
	 * Carrega el selector de productes disponibles.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 */
	public void loadProductesSource(Integer idProductor) {
		pService.getAllProductesSource(idProductor, new AsyncCallback<List<Producte>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els productes disponibles", arg0);
			}

			@Override
			public void onSuccess(List<Producte> productes) {
				productesSource.removeAll();
				productesSource.add(productes);
			}
		});
	}

	/**
	 * Carrega el selector de productes associats.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 */
	public void loadProductesTarget(Integer idProductor) {
		pService.getAllProductesTarget(idProductor, new AsyncCallback<List<Producte>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els productes seleccionats", arg0);
			}

			@Override
			public void onSuccess(List<Producte> productes) {
				productesTarget.removeAll();
				productesTarget.add(productes);
			}
		});
	}

	@Override
	public void onCancel() {
		getGrid().getSelectionModel().deselectAll();
		loadEsdevenimentsSource(null);
		loadEsdevenimentsTarget(null);
		loadProductesSource(null);
		loadProductesTarget(null);
		tipusProductor.clearSelections();
		comarca.clearSelections();
		poblacio.clearSelections();
		esdeveniments.setEnabled(false);
		productes.setEnabled(false);
		mapaManteniment.setFeatureId(0);
	}
	
	@Override
	protected void onUnload() {
		onCancel();				
		super.onUnload();
	}
	
	protected void refresh() {
		formAndMapPanel.remove(mapaManteniment);
		mapaManteniment = new MapaMantenimentLayout("productor", "images/orange-dot.png");
		mapaManteniment.enableCreate();
		this.mapaManteniment.setHeading(getMessages().title_productor_localitzacio());
		formAndMapPanel.add(mapaManteniment, new RowData(0.6, 1, new Margins(5, 5, 5, 5)));
		formAndMapPanel.layout();
		super.onAfterLayout();
	}

}
