package uoc.edu.client.layout;

import java.util.ArrayList;
import java.util.List;

import org.gwtopenmaps.openlayers.client.Bounds;
import org.gwtopenmaps.openlayers.client.Icon;
import org.gwtopenmaps.openlayers.client.LonLat;
import org.gwtopenmaps.openlayers.client.MapOptions;
import org.gwtopenmaps.openlayers.client.MapWidget;
import org.gwtopenmaps.openlayers.client.Marker;
import org.gwtopenmaps.openlayers.client.Size;
import org.gwtopenmaps.openlayers.client.control.NavToolbar;
import org.gwtopenmaps.openlayers.client.control.PanZoomBar;
import org.gwtopenmaps.openlayers.client.control.PanZoomBarOptions;
import org.gwtopenmaps.openlayers.client.event.MarkerBrowserEventListener;
import org.gwtopenmaps.openlayers.client.layer.GMapType;
import org.gwtopenmaps.openlayers.client.layer.Google;
import org.gwtopenmaps.openlayers.client.layer.GoogleOptions;
import org.gwtopenmaps.openlayers.client.layer.Layer;
import org.gwtopenmaps.openlayers.client.layer.Markers;
import org.gwtopenmaps.openlayers.client.layer.OSM;
import org.gwtopenmaps.openlayers.client.layer.OSMImpl;
import org.gwtopenmaps.openlayers.client.layer.OSMOptions;
import org.gwtopenmaps.openlayers.client.layer.TransitionEffect;
import org.gwtopenmaps.openlayers.client.layer.WMS;
import org.gwtopenmaps.openlayers.client.layer.WMSOptions;
import org.gwtopenmaps.openlayers.client.layer.WMSParams;

import uoc.edu.client.Messages;
import uoc.edu.client.openlayers.OSMExtended;
import uoc.edu.client.service.ComarcaServiceAsync;
import uoc.edu.client.service.EsdevenimentServiceAsync;
import uoc.edu.client.service.PoblacioServiceAsync;
import uoc.edu.client.service.ProducteServiceAsync;
import uoc.edu.client.service.SearchServiceAsync;
import uoc.edu.client.service.ServiceConstants;
import uoc.edu.client.service.TipusEsdevenimentServiceAsync;
import uoc.edu.client.service.TipusProductorServiceAsync;
import uoc.edu.client.service.XcsServiceAsync;
import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.Producte;
import uoc.edu.shared.model.SearchConfig;
import uoc.edu.shared.model.SearchResult;
import uoc.edu.shared.model.TipusEsdeveniment;
import uoc.edu.shared.model.TipusProductor;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.Registry;
import com.extjs.gxt.ui.client.Style.HorizontalAlignment;
import com.extjs.gxt.ui.client.Style.Orientation;
import com.extjs.gxt.ui.client.event.ButtonEvent;
import com.extjs.gxt.ui.client.event.SelectionListener;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.util.Margins;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Label;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.button.Button;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.layout.RowData;
import com.extjs.gxt.ui.client.widget.layout.RowLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Image;

/**
 * Pantalla de cerca de xcs, productors i esdeveniments.
 * 
 * @author Jordi Farré
 * 
 */
public class SearchLayout extends ContentPanel {

	/** Marcadors que estem mostrant en el mapa */
	private List<Marker> markers = new ArrayList<Marker>();

	/** Capa per mostrar els marcadors */
	private Markers markersLayer;

	/** Missatges i18n */
	private Messages messages;

	/** Servei per recuperar les dades de les comarques */
	private ComarcaServiceAsync comarcaService;

	/** Servei per recuperar les dades de les poblacions */
	private PoblacioServiceAsync poblacioService;

	/** Servei per recuperar les dades d'esdeveniments */
	private EsdevenimentServiceAsync esdevenimentService;

	/** Servei per recuperar les dades dels tipus d'esdeveniments */
	private TipusEsdevenimentServiceAsync teService;

	/** Servei per recuperar les dades dels tipus de productors */
	private TipusProductorServiceAsync tpService;

	/** Servei per recuperar les dades de productes */
	private ProducteServiceAsync producteService;

	/** Servei per recuperar les dades de xarxes de consum solidari */
	private XcsServiceAsync xcsService;

	/** Servei per la cerca de informació */
	private SearchServiceAsync searchService;

	/** Panell que contindrà el mapa */
	private ContentPanel mapPanel;

	/** Widget que utilitzem per mostrar el mapa */
	private MapWidget mapWidget;

	/** Layer wms per el mostrar informacio geografica */
	private WMS wmsLayer;
	
	/** Capa base de tipus osm */
	private OSMExtended osm;

	/** Panell que contindrà el formulari i la llegenda */
	private ContentPanel formAndLegendPanel;

	/** Panell amb el formulari de criteris de cerca */
	private FormPanel formPanel;

	/** Panell que contindrà la llegenda */
	private ContentPanel legendPanel;

	/** Camp comarca del formulari */
	private ComboBox<Comarca> comarca;

	/** Camp poblacio del formulari */
	private ComboBox<Poblacio> poblacio;

	/** Camp producte del formulari */
	private ComboBox<Producte> producte;

	/** Camp esdeveniment del formulari */
	private ComboBox<Esdeveniment> esdeveniment;

	/** Camp tipus esdeveniment del formulari */
	private ComboBox<TipusEsdeveniment> tipusEsdeveniment;

	/** Camp tipus productor del formulari */
	private ComboBox<TipusProductor> tipusProductor;

	/** Camp xcs del formulari */
	private ComboBox<XCS> xcs;

	/** Conté la informació de les comarques */
	private ListStore<Comarca> comarques;

	/** Conté la informació de les poblacions */
	private ListStore<Poblacio> poblacions;

	/** Conté la informació dels productes */
	private ListStore<Producte> productes;

	/** Conté la informació dels esdeveniments */
	private ListStore<Esdeveniment> esdeveniments;

	/** Conté la informació dels tipus d'esdeveniments */
	private ListStore<TipusEsdeveniment> tipusEsdeveniments;

	/** Conté la informació dels tipus de productors */
	private ListStore<TipusProductor> tipusProductors;

	/** Conté la informació dels xcs */
	private ListStore<XCS> xcss;

	@Override
	protected void onRender(Element parent, int pos) {
		this.setHeaderVisible(false);
		super.onRender(parent, pos);
		// Creem el servei i18n
		messages = GWT.create(Messages.class);
		// Recuperem els serveis rpc
		comarcaService = (ComarcaServiceAsync) Registry.get(ServiceConstants.COMARCA_SERVICE);
		poblacioService = (PoblacioServiceAsync) Registry.get(ServiceConstants.POBLACIO_SERVICE);
		searchService = (SearchServiceAsync) Registry.get(ServiceConstants.SEARCH_SERVICE);
		teService = (TipusEsdevenimentServiceAsync) Registry.get(ServiceConstants.TIPUSESDEVENIMENT_SERVICE);
		tpService = (TipusProductorServiceAsync) Registry.get(ServiceConstants.TIPUSPRODUCTOR_SERVICE);
		producteService = (ProducteServiceAsync) Registry.get(ServiceConstants.PRODUCTE_SERVICE);
		xcsService = (XcsServiceAsync) Registry.get(ServiceConstants.XCS_SERVICE);
		esdevenimentService = (EsdevenimentServiceAsync) Registry.get(ServiceConstants.ESDEVENIMENT_SERVICE);
		// Recuperem la informació per els combobox
		comarcaService.getAll(new AsyncCallback<List<Comarca>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<Comarca> comarcaList) {
				Comarca all = new Comarca();
				all.setId(Comarca.TOTS);
				all.setDescripcio(messages.combobox_totes());
				comarques.add(all);
				comarques.add(comarcaList);
				comarca.setValue(all);
			}
		});
		poblacioService.getAll(new AsyncCallback<List<Poblacio>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les poblacions", arg0);
			}

			@Override
			public void onSuccess(List<Poblacio> poblacioList) {
				Poblacio all = new Poblacio();
				all.setId(Poblacio.TOTS);
				all.setDescripcio(messages.combobox_totes());
				poblacions.add(all);
				poblacions.add(poblacioList);
				poblacio.setValue(all);
			}
		});
		producteService.getAll(new AsyncCallback<List<Producte>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els productes", arg0);
			}

			@Override
			public void onSuccess(List<Producte> producteList) {
				Producte all = new Producte();
				all.setId(Producte.TOTS);
				all.setDescripcio(messages.combobox_tots());
				productes.add(all);
				productes.add(producteList);
				producte.setValue(all);
			}
		});
		teService.getAll(new AsyncCallback<List<TipusEsdeveniment>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els tipus de esdeveniments", arg0);
			}

			@Override
			public void onSuccess(List<TipusEsdeveniment> tipusEsdevenimentsList) {
				TipusEsdeveniment all = new TipusEsdeveniment();
				all.setId(TipusEsdeveniment.TOTS);
				all.setDescripcio(messages.combobox_tots());
				tipusEsdeveniments.add(all);
				tipusEsdeveniments.add(tipusEsdevenimentsList);
				tipusEsdeveniment.setValue(all);
			}
		});
		esdevenimentService.getAll(new AsyncCallback<List<Esdeveniment>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar esdeveniments", arg0);
			}

			@Override
			public void onSuccess(List<Esdeveniment> esdevenimentsList) {
				Esdeveniment all = new Esdeveniment();
				all.setId(Esdeveniment.TOTS);
				all.setDescripcio(messages.combobox_tots());
				esdeveniments.add(all);
				esdeveniments.add(esdevenimentsList);
				esdeveniment.setValue(all);
			}
		});
		tpService.getAll(new AsyncCallback<List<TipusProductor>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els tipus de productors", arg0);
			}

			@Override
			public void onSuccess(List<TipusProductor> tipusProductorsList) {
				TipusProductor all = new TipusProductor();
				all.setId(TipusEsdeveniment.TOTS);
				all.setDescripcio(messages.combobox_tots());
				tipusProductors.add(all);
				tipusProductors.add(tipusProductorsList);
				tipusProductor.setValue(all);
			}
		});
		xcsService.getAll(new AsyncCallback<List<XCS>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les xcs", arg0);
			}

			@Override
			public void onSuccess(List<XCS> xcsList) {
				XCS all = new XCS();
				all.setId(XCS.TOTS);
				all.setDescripcio(messages.combobox_tots());
				xcss.add(all);
				xcss.add(xcsList);
				xcs.setValue(all);
			}
		});
		this.setLayout(new RowLayout(Orientation.HORIZONTAL));
		createMapPanel();
		createFormPanel();
		this.setHeight(parent.getClientHeight());
		this.add(mapPanel, new RowData(0.7, 1, new Margins(5, 5, 5, 5)));
		this.add(formAndLegendPanel, new RowData(0.3, 1, new Margins(5, 5, 5, 5)));
	}

	/**
	 * Crea el panell per el mapa.
	 */
	private void createMapPanel() {
		mapPanel = new ContentPanel();
		mapPanel.setFrame(true);
		mapPanel.setHeading(messages.title_cerca_resultats());
		// Definim les opcions per defecte del mapa
		Bounds bounds = new Bounds(12746, 4932506, 374628, 5295759);
		MapOptions defaultMapOptions = new MapOptions();
		defaultMapOptions.removeDefaultControls();
		defaultMapOptions.setProjection("EPSG:900913");		
		defaultMapOptions.setMaxExtent(bounds);
		defaultMapOptions.setMaxResolutionToAuto();
		// Instanciem el widget
		mapWidget = new MapWidget("100%", "100%", defaultMapOptions);
		// Definim la capa base de tipus osm
		osm = new OSMExtended("osm");
		// Definim els parametres del wms
		WMSParams wmsParams = new WMSParams();
		wmsParams.setFormat("image/png");
		wmsParams.setLayers("xcs:planet_osm_line");
		wmsParams.setIsTransparent(true);		
		// Definim les opcions del layer
		WMSOptions wmsLayerParams = new WMSOptions();
		wmsLayerParams.setUntiled();	
		wmsLayerParams.setLayerOpacity(0.4);
		wmsLayerParams.setTransitionEffect(TransitionEffect.RESIZE);		
		// Definim el layer que crida al wms de geoserver
		wmsLayer = new WMS("xcs:planet_osm_line", "http://localhost:8080/geoserver/wms", wmsParams, wmsLayerParams);		
		// Definim el layer que ens permetra afegir marques en el mapa
		markersLayer = new Markers("markers");

		// markersLayer.removeMarker(marker);
		// Afegim els controls necessaris i seleccionem un centre per el mapa
		mapWidget.getMap().addControl(new PanZoomBar());
		mapWidget.getMap().addControl(new NavToolbar());
		mapWidget.getMap().addLayers(new Layer[] {osm, wmsLayer, markersLayer });
		mapWidget.getMap().setCenter(new LonLat(238408, 5069840), 12);
		// Afegim el widget del mapa al panell creat
		mapPanel.add(mapWidget);
	}

	/**
	 * Crea el panell per el formulari amb els criteris de cerca.
	 */
	private void createFormPanel() {
		formAndLegendPanel = new ContentPanel(new RowLayout(Orientation.VERTICAL));
		formAndLegendPanel.setHeaderVisible(false);
		formPanel = new FormPanel();
		// Camp comarca
		comarques = new ListStore<Comarca>();
		comarca = new ComboBox<Comarca>();
		comarca.setStore(comarques);
		comarca.setFieldLabel(messages.columna_comarca());
		comarca.setValueField("idcomarca");
		comarca.setDisplayField("comarcadesc");
		comarca.setTriggerAction(TriggerAction.ALL);
		comarca.setEditable(false);
		formPanel.add(comarca);
		// Camp poblacio
		poblacions = new ListStore<Poblacio>();
		poblacio = new ComboBox<Poblacio>();
		poblacio.setStore(poblacions);
		poblacio.setFieldLabel(messages.columna_poblacio());
		poblacio.setValueField("idpoblacio");
		poblacio.setDisplayField("poblaciodesc");
		poblacio.setTriggerAction(TriggerAction.ALL);
		poblacio.setEditable(false);
		formPanel.add(poblacio);
		// Camp producte
		productes = new ListStore<Producte>();
		producte = new ComboBox<Producte>();
		producte.setStore(productes);
		producte.setFieldLabel(messages.columna_producte());
		producte.setValueField("idproducte");
		producte.setDisplayField("productedesc");
		producte.setTriggerAction(TriggerAction.ALL);
		producte.setEditable(false);
		formPanel.add(producte);
		// Camp esdeveniment
		esdeveniments = new ListStore<Esdeveniment>();
		esdeveniment = new ComboBox<Esdeveniment>();
		esdeveniment.setStore(esdeveniments);
		esdeveniment.setFieldLabel(messages.menu_esdeveniment());
		esdeveniment.setValueField("idesdeveniment");
		esdeveniment.setDisplayField("descripcio");
		esdeveniment.setTriggerAction(TriggerAction.ALL);
		esdeveniment.setEditable(false);
		formPanel.add(esdeveniment);
		// Camp tipus esdeveniment
		tipusEsdeveniments = new ListStore<TipusEsdeveniment>();
		tipusEsdeveniment = new ComboBox<TipusEsdeveniment>();
		tipusEsdeveniment.setStore(tipusEsdeveniments);
		tipusEsdeveniment.setFieldLabel(messages.columna_tipusEsdeveniment_abrev());
		tipusEsdeveniment.setValueField("idtipusesdeveniment");
		tipusEsdeveniment.setDisplayField("tipusesdevenimentdesc");
		tipusEsdeveniment.setTriggerAction(TriggerAction.ALL);
		tipusEsdeveniment.setEditable(false);
		formPanel.add(tipusEsdeveniment);
		// Camp tipus productors
		tipusProductors = new ListStore<TipusProductor>();
		tipusProductor = new ComboBox<TipusProductor>();
		tipusProductor.setStore(tipusProductors);
		tipusProductor.setFieldLabel(messages.columna_tipusProductor_abrev());
		tipusProductor.setValueField("idtipusproductor");
		tipusProductor.setDisplayField("tipusproductordesc");
		tipusProductor.setTriggerAction(TriggerAction.ALL);
		tipusProductor.setEditable(false);
		formPanel.add(tipusProductor);
		// Camp xcs
		// Camp tipus productors
		xcss = new ListStore<XCS>();
		xcs = new ComboBox<XCS>();
		xcs.setStore(xcss);
		xcs.setFieldLabel(messages.columna_xcs());
		xcs.setValueField("idxcs");
		xcs.setDisplayField("descripcio");
		xcs.setTriggerAction(TriggerAction.ALL);
		xcs.setEditable(false);
		formPanel.add(xcs);
		// Botó buscar
		Button button = new Button(messages.boto_buscar());
		button.addSelectionListener(new SelectionListener<ButtonEvent>() {
			@Override
			public void componentSelected(ButtonEvent ce) {
				SearchConfig searchConfig = new SearchConfig();
				searchConfig.setComarca(comarca.getValue());
				searchConfig.setPoblacio(poblacio.getValue());
				searchConfig.setProducte(producte.getValue());
				searchConfig.setTipusEsdeveniment(tipusEsdeveniment.getValue());
				searchConfig.setTipusProductor(tipusProductor.getValue());
				searchConfig.setXcs(xcs.getValue());
				searchConfig.setEsdeveniment(esdeveniment.getValue());
				searchService.search(searchConfig, new AsyncCallback<List<SearchResult>>() {
					@Override
					public void onFailure(Throwable arg0) {
						MessageBox.alert(messages.dialeg_error(), messages.search_ko(), null);
					}

					@Override
					public void onSuccess(List<SearchResult> newMarkers) {
						updateMarkers(newMarkers);
					}
				});
			}
		});
		formPanel.add(button);
		formPanel.setButtonAlign(HorizontalAlignment.CENTER);
		formPanel.setHeading(messages.title_cerca_criteris());
		formPanel.setLabelWidth(95);
		formPanel.setFrame(true);
		formAndLegendPanel.add(formPanel, new RowData(1, 0.65, new Margins(5, 5, 5, 5)));
		// Creem el panell per la llegenda
		legendPanel = new ContentPanel(new RowLayout(Orientation.VERTICAL));
		legendPanel.setFrame(true);
		// Afegim el contingut de la llegenda
		Label xcsLabel = new Label(messages.menu_xcs());
		Image xcsImage = new Image("images/red-dot.png");
		ContentPanel xcsPanel = new ContentPanel();
		xcsPanel.add(xcsImage);
		xcsPanel.add(xcsLabel);
		xcsPanel.setHeaderVisible(false);
		xcsPanel.setBodyBorder(false);
		xcsPanel.setBodyStyle("red-dot");
		Label productorsLabel = new Label(messages.menu_productor());
		Image productorsImage = new Image("images/orange-dot.png");
		ContentPanel productorsPanel = new ContentPanel();
		productorsPanel.add(productorsImage);
		productorsPanel.add(productorsLabel);
		productorsPanel.setHeaderVisible(false);
		productorsPanel.setBodyBorder(false);
		Label esdevenimentsLabel = new Label(messages.menu_esdeveniment());
		Image esdevenimentsImage = new Image("images/green-dot.png");
		ContentPanel esdevenimentsPanel = new ContentPanel();
		esdevenimentsPanel.add(esdevenimentsImage);
		esdevenimentsPanel.add(esdevenimentsLabel);
		esdevenimentsPanel.setHeaderVisible(false);
		esdevenimentsPanel.setBodyBorder(false);
		legendPanel.add(xcsPanel, new RowData(1, 0.33, new Margins(5, 2, 0, 5)));
		legendPanel.add(productorsPanel, new RowData(1, 0.33, new Margins(0, 2, 0, 5)));
		legendPanel.add(esdevenimentsPanel, new RowData(1, 0.33, new Margins(0, 2, 0, 5)));
		legendPanel.setHeading(messages.title_cerca_llegenda());
		formAndLegendPanel.add(legendPanel, new RowData(1, 0.35, new Margins(5, 5, 5, 5)));
	}

	/**
	 * Actualitza els marcadors en el mapa.
	 * 
	 * @param newMarkers
	 *            Llista de nous marcadors.
	 */
	private void updateMarkers(List<SearchResult> newMarkers) {
		// Primer esborrem els marcadors que estem mostrant actualment
		for (Marker marker : this.markers) {
			markersLayer.removeMarker(marker);
		}
		// Ara afegim els nous marcadors
		this.markers.clear();
		if (newMarkers != null && !newMarkers.isEmpty()) {
			for (SearchResult searchResult : newMarkers) {
				LonLat newLonLat = new LonLat(searchResult.getLon(), searchResult.getLat());
				Marker marker = new Marker(newLonLat);
				final int tipus = searchResult.getTipus();
				final String descripcio = searchResult.getDescripcio();
				if (SearchResult.XCS == tipus) {
					marker.setIcon(new Icon("images/red-dot.png", new Size(25, 25)));
				} else if (SearchResult.PRODUCTOR == tipus) {
					marker.setIcon(new Icon("images/orange-dot.png", new Size(25, 25)));
				} else if (SearchResult.ESDEVENIMENT == tipus) {
					marker.setIcon(new Icon("images/green-dot.png", new Size(25, 25)));
				}
				marker.addBrowserEventListener("mousedown", new MarkerBrowserEventListener() {
					@Override
					public void onBrowserEvent(MarkerBrowserEvent markerBrowserEvent) {
						String title = null;
						if (SearchResult.XCS == tipus) {
							title = messages.menu_xcs();
						} else if (SearchResult.PRODUCTOR == tipus) {
							title = messages.menu_productor();
						} else if (SearchResult.ESDEVENIMENT == tipus) {
							title = messages.menu_esdeveniment();
						}
						MessageBox.info(title, descripcio, null);
					}
				});

				this.markers.add(marker);
				this.markersLayer.addMarker(marker);
			}
		} else {
			MessageBox.alert(messages.dialeg_error(), messages.search_ko(), null);
		}
	}

}
