package uoc.edu.client.layout;

import java.util.ArrayList;
import java.util.List;

import uoc.edu.client.service.ServiceConstants;
import uoc.edu.client.service.TipusEsdevenimentService;
import uoc.edu.client.service.TipusEsdevenimentServiceAsync;
import uoc.edu.shared.model.TipusEsdeveniment;

import com.extjs.gxt.ui.client.Registry;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.RpcProxy;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.util.Margins;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.layout.RowData;
import com.extjs.gxt.ui.client.widget.layout.RowLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Pantalla de manteniment de tipus de esdeveniment.
 * 
 * @author Jordi Farré
 * 
 */
public class TipusEsdevenimentLayout extends MantenimentLayout<TipusEsdeveniment> {

	/** Servei web per el manteniment de tipus de esdeveniments */
	private TipusEsdevenimentServiceAsync teService;

	/** Camp id del formulari */
	private NumberField id;

	/** Camp descripció del formulari */
	private TextField<String> descripcio;

	@Override
	protected void onRender(Element parent, int pos) {
		getGrid().setStateId("tipusEsdeveniment");
		setHeaderVisible(false);
		super.onRender(parent, pos);
		// Recuperem els serveis rpc
		teService = (TipusEsdevenimentServiceAsync) Registry.get(ServiceConstants.TIPUSESDEVENIMENT_SERVICE);
		setLayout(new RowLayout());
		setHeight(parent.getClientHeight());
		add(getGridPanel(), new RowData(1, 0.5, new Margins(5, 5, 5, 5)));
		add(getFormPanel(), new RowData(1, 0.5, new Margins(5, 5, 5, 5)));
		this.getFormPanel().setHeading(getMessages().title_tipusEsdeveniment_dades());
	}

	@Override
	public List<ColumnConfig> createColumnConfig() {
		List<ColumnConfig> columnes = new ArrayList<ColumnConfig>();
		columnes.add(new ColumnConfig("idtipusesdeveniment", getMessages().columna_id(), 200));
		columnes.add(new ColumnConfig("tipusesdevenimentdesc", getMessages().columna_descripcio(), 500));
		return columnes;
	}

	@Override
	public List<Field> createFormFields() {
		List<Field> fields = new ArrayList<Field>();
		// Camp id
		id = new NumberField();
		id.setName("idtipusesdeveniment");
		id.setFieldLabel(getMessages().columna_id());
		id.setAllowDecimals(false);
		id.setReadOnly(true);
		fields.add(id);
		// Camp descripció
		descripcio = new TextField<String>();
		descripcio.setName("tipusesdevenimentdesc");
		descripcio.setFieldLabel(getMessages().columna_descripcio());
		descripcio.setAllowBlank(false);
		fields.add(descripcio);
		return fields;
	}

	@Override
	public RpcProxy<PagingLoadResult<TipusEsdeveniment>> createPaginationProxy() {
		// Creem un proxy de paginació que crida al nostre servei de tipus de
		// esdeveniment
		RpcProxy<PagingLoadResult<TipusEsdeveniment>> proxy = new RpcProxy<PagingLoadResult<TipusEsdeveniment>>() {
			@Override
			protected void load(Object loadConfig, AsyncCallback<PagingLoadResult<TipusEsdeveniment>> callback) {
				teService.getAll((PagingLoadConfig) loadConfig, callback);
			}
		};
		return proxy;
	}

	@Override
	public void onSave(FormPanel formPanel) {
		if (formPanel != null) {
			// Recuperem les dades del formulari
			TipusEsdeveniment saveObject = new TipusEsdeveniment();
			if (id.getValue() != null) {
				saveObject.setId(id.getValue().intValue());
			}
			saveObject.setDescripcio(descripcio.getValue());
			// Guardem el tipus de esdeveniment
			teService.save(saveObject, new AsyncCallback<Void>() {
				@Override
				public void onFailure(Throwable arg0) {
					// En cas de error mostrem un missatge generic
					MessageBox.alert(getMessages().dialeg_error(), getMessages().guardar_ko(), null);
					getFormPanel().clear();
				}

				@Override
				public void onSuccess(Void arg0) {
					// En cas de guardar-se correctament enviem un event al grid
					// perque es recarregui.
					getGrid().fireEvent(Events.Attach);
					getFormPanel().clear();
				}
			});
		}
	}

	@Override
	public void onDelete(TipusEsdeveniment deleteObject) {
		if (deleteObject != null) {
			// Intentem esborrar el tipus de esdeveniment
			teService.delete(deleteObject, new AsyncCallback<Void>() {
				@Override
				public void onFailure(Throwable arg0) {
					// En cas de error mostrem un missatge generic de error
					MessageBox.alert(getMessages().dialeg_error(), getMessages().esborrar_ko(), null);
				}

				@Override
				public void onSuccess(Void arg0) {
					// En cas de esborrar-se correctament enviem un event al
					// grid perque
					// es recarregui
					getGrid().fireEvent(Events.Attach);
				}
			});
		}
	}
	
	@Override
	public void onCancel() {
		// TODO Auto-generated method stub
		
	}

}
