package uoc.edu.client.layout;

import java.util.ArrayList;
import java.util.List;

import org.gwtopenmaps.openlayers.client.feature.VectorFeature;
import org.gwtopenmaps.openlayers.client.util.Attributes;

import uoc.edu.client.service.ComarcaServiceAsync;
import uoc.edu.client.service.PoblacioServiceAsync;
import uoc.edu.client.service.ServiceConstants;
import uoc.edu.client.service.XcsServiceAsync;
import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Poblacio;
import uoc.edu.shared.model.Productor;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.Registry;
import com.extjs.gxt.ui.client.Style.Orientation;
import com.extjs.gxt.ui.client.Style.Scroll;
import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.extjs.gxt.ui.client.data.RpcProxy;
import com.extjs.gxt.ui.client.event.Events;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.event.SelectionChangedEvent;
import com.extjs.gxt.ui.client.store.ListStore;
import com.extjs.gxt.ui.client.util.Margins;
import com.extjs.gxt.ui.client.widget.ContentPanel;
import com.extjs.gxt.ui.client.widget.Info;
import com.extjs.gxt.ui.client.widget.MessageBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox;
import com.extjs.gxt.ui.client.widget.form.ComboBox.TriggerAction;
import com.extjs.gxt.ui.client.widget.form.DualListField;
import com.extjs.gxt.ui.client.widget.form.DualListField.Mode;
import com.extjs.gxt.ui.client.widget.form.Field;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.form.ListField;
import com.extjs.gxt.ui.client.widget.form.NumberField;
import com.extjs.gxt.ui.client.widget.form.TextField;
import com.extjs.gxt.ui.client.widget.grid.ColumnConfig;
import com.extjs.gxt.ui.client.widget.layout.RowData;
import com.extjs.gxt.ui.client.widget.layout.RowLayout;
import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Element;
import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * Pantalla de manteniment de xcs.
 * 
 * @author Jordi Farré
 * 
 */
public class XcsLayout extends MantenimentLayout<XCS> {

	/** Servei web per el manteniment de xcs */
	private XcsServiceAsync xService;

	/** Servei per recuperar les dades de les comarques */
	private ComarcaServiceAsync comarcaService;

	/** Servei per recuperar les dades de les poblacions */
	private PoblacioServiceAsync poblacioService;

	/** Camp id del formulari */
	private NumberField id;

	/** Camp adreça del formulari */
	private TextField<String> adreca;

	/** Camp descripció del formulari */
	private TextField<String> descripcio;

	/** Camp teléfon del formulari */
	private TextField<String> telefon;

	/** Camp email del formulari */
	private TextField<String> email;

	/** Camp comarca del formulari */
	private ComboBox<Comarca> comarca;

	/** Camp poblacio del formulari */
	private ComboBox<Poblacio> poblacio;

	/** Selecció de productors pel formulari */
	private DualListField<Productor> productors;

	/** Conté la informació de les comarques */
	private ListStore<Comarca> comarques;

	/** Conté la informació de les poblacions */
	private ListStore<Poblacio> poblacions;

	/** Productors disponibles */
	private ListStore<Productor> productorsSource;

	/** Productors seleccionats */
	private ListStore<Productor> productorsTarget;

	/** Instancia del mapa per mostrar els productors */
	private MapaMantenimentLayout mapaManteniment;
	
	/** Panell que contindrà el formulari i el mapa */
	ContentPanel formAndMapPanel;

	@Override
	protected void onRender(Element parent, int pos) {
		getGrid().setStateId("xcs");
		setHeaderVisible(false);
		super.onRender(parent, pos);
		// Recuperem els serveis rpc
		xService = (XcsServiceAsync) Registry.get(ServiceConstants.XCS_SERVICE);
		comarcaService = (ComarcaServiceAsync) Registry.get(ServiceConstants.COMARCA_SERVICE);
		poblacioService = (PoblacioServiceAsync) Registry.get(ServiceConstants.POBLACIO_SERVICE);
		// Recuperem la informació per els combobox
		comarcaService.getAll(new AsyncCallback<List<Comarca>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les comarques", arg0);
			}

			@Override
			public void onSuccess(List<Comarca> comarcaList) {
				comarques.add(comarcaList);
			}
		});
		poblacioService.getAll(new AsyncCallback<List<Poblacio>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar les poblacions", arg0);
			}

			@Override
			public void onSuccess(List<Poblacio> poblacioList) {
				poblacions.add(poblacioList);
			}
		});
		loadProductorsSource(null);
		loadProductorsTarget(null);
		getGrid().getSelectionModel().addListener(Events.SelectionChange,
				new Listener<SelectionChangedEvent<XCS>>() {
					@Override
					public void handleEvent(SelectionChangedEvent<XCS> be) {
						if (be.getSelection().size() > 0) {
							loadProductorsSource(be.getSelectedItem().getId());
							loadProductorsTarget(be.getSelectedItem().getId());
							comarca.setValue(be.getSelectedItem().getComarca());
							poblacio.setValue(be.getSelectedItem().getPoblacio());
							productors.setEnabled(true);
							mapaManteniment.setFeatureId(be.getSelectedItem().getId());
						} else {
							loadProductorsSource(null);
							loadProductorsTarget(null);
							comarca.clearSelections();
							poblacio.clearSelections();
							productors.setEnabled(false);
							mapaManteniment.setFeatureId(0);
						}

					}
				});
		setLayout(new RowLayout());
		setHeight(parent.getClientHeight());
		add(getGridPanel(), new RowData(1, 0.3, new Margins(5, 5, 5, 5)));

		formAndMapPanel = new ContentPanel();
		formAndMapPanel.setHeaderVisible(false);
		formAndMapPanel.setLayout(new RowLayout(Orientation.HORIZONTAL));
		FormPanel formPanel = getFormPanel();
		formPanel.setLabelWidth(150);
		formPanel.setScrollMode(Scroll.AUTOY);
		formAndMapPanel.add(formPanel, new RowData(0.4, 1, new Margins(5, 5, 5, 5)));
		add(formAndMapPanel, new RowData(1, 0.7, new Margins(5, 5, 5, 5)));
		this.getFormPanel().setHeading(getMessages().title_xcs_dades());
	}

	@Override
	public List<ColumnConfig> createColumnConfig() {
		List<ColumnConfig> columnes = new ArrayList<ColumnConfig>();
		columnes.add(new ColumnConfig("idxcs", getMessages().columna_id(), 50));
		columnes.add(new ColumnConfig("adreca", getMessages().columna_adreca(), 250));
		columnes.add(new ColumnConfig("descripcio", getMessages().columna_descripcio(), 200));
		columnes.add(new ColumnConfig("telefon", getMessages().columna_descripcio(), 100));
		columnes.add(new ColumnConfig("email", getMessages().columna_descripcio(), 100));
		columnes.add(new ColumnConfig("comarcadesc", getMessages().columna_comarca(), 150));
		columnes.add(new ColumnConfig("poblaciodesc", getMessages().columna_poblacio(), 200));
		return columnes;
	}

	@Override
	public List<Field> createFormFields() {
		List<Field> fields = new ArrayList<Field>();
		// Camp id
		id = new NumberField();
		id.setName("idxcs");
		id.setFieldLabel(getMessages().columna_id());
		id.setAllowDecimals(false);
		id.setReadOnly(true);
		fields.add(id);
		// Camp adreça
		adreca = new TextField<String>();
		adreca.setName("adreca");
		adreca.setFieldLabel(getMessages().columna_adreca());
		fields.add(adreca);
		// Camp descripció
		descripcio = new TextField<String>();
		descripcio.setName("descripcio");
		descripcio.setFieldLabel(getMessages().columna_descripcio());
		descripcio.setAllowBlank(false);
		fields.add(descripcio);
		// Camp telefon
		telefon = new TextField<String>();
		telefon.setName("telefon");
		telefon.setFieldLabel(getMessages().columna_telefon());
		fields.add(telefon);
		// Camp descripció
		email = new TextField<String>();
		email.setName("email");
		email.setFieldLabel(getMessages().columna_email());
		fields.add(email);
		// Camp comarca
		comarques = new ListStore<Comarca>();
		comarca = new ComboBox<Comarca>();
		comarca.setStore(comarques);
		comarca.setFieldLabel(getMessages().columna_comarca());
		comarca.setValueField("idcomarca");
		comarca.setDisplayField("comarcadesc");
		comarca.setEditable(false);
		comarca.setAllowBlank(false);
		comarca.setTriggerAction(TriggerAction.ALL);
		fields.add(comarca);
		// Camp poblacio
		poblacions = new ListStore<Poblacio>();
		poblacio = new ComboBox<Poblacio>();
		poblacio.setStore(poblacions);
		poblacio.setFieldLabel(getMessages().columna_poblacio());
		poblacio.setValueField("idpoblacio");
		poblacio.setDisplayField("poblaciodesc");
		poblacio.setEditable(false);
		poblacio.setAllowBlank(false);
		poblacio.setTriggerAction(TriggerAction.ALL);
		fields.add(poblacio);
		// Control per seleccionar els productors
		productors = new DualListField<Productor>();
		productors.setMode(Mode.INSERT);
		productors.setFieldLabel(getMessages().columna_productors());
		productorsSource = new ListStore<Productor>();
		productorsTarget = new ListStore<Productor>();
		productors.setEnabled(false);
		ListField<Productor> source = productors.getFromList();
		source.setStore(productorsSource);
		source.setDisplayField("descripcio");
		ListField<Productor> target = productors.getToList();
		target.setStore(productorsTarget);
		target.setDisplayField("descripcio");
		fields.add(productors);
		return fields;
	}

	@Override
	public RpcProxy<PagingLoadResult<XCS>> createPaginationProxy() {
		// Creem un proxy de paginació que crida al nostre servei de
		// xcs
		RpcProxy<PagingLoadResult<XCS>> proxy = new RpcProxy<PagingLoadResult<XCS>>() {
			@Override
			protected void load(Object loadConfig, AsyncCallback<PagingLoadResult<XCS>> callback) {
				xService.getAll((PagingLoadConfig) loadConfig, callback);
			}
		};
		return proxy;
	}

	@Override
	public void onSave(FormPanel formPanel) {
		if (formPanel != null) {
			// Recuperem les dades del formulari
			VectorFeature feature = mapaManteniment.getFeature(); 
			if (feature != null) {
				Attributes attributes = new Attributes();
				attributes.setAttribute("adreca", adreca.getValue());
				attributes.setAttribute("descripcio", descripcio.getValue());
				attributes.setAttribute("telefon", telefon.getValue());
				attributes.setAttribute("email", email.getValue());
				attributes.setAttribute("idcomarca", comarca.getValue().getId());
				attributes.setAttribute("idpoblacio", poblacio.getValue().getId());
				feature.setAttributes(attributes);
				mapaManteniment.getSaveStrategy().save();
				// Guardem els productors
				xService.saveProductors(productors.getFromList().getStore().getModels(), productors.getToList().getStore().getModels(), id.getValue().intValue(), new AsyncCallback<Void>() {
					@Override
					public void onFailure(Throwable arg0) {
						// En cas de error mostrem un missatge generic
						MessageBox.alert(getMessages().dialeg_error(), getMessages().guardar_ko(), null);
					}

					@Override
					public void onSuccess(Void arg0) {
					}
				});
				getGrid().fireEvent(Events.Attach);
				mapaManteniment.setFeatureId(0);
				getFormPanel().clear();
			} else {
				MessageBox.alert(getMessages().dialeg_error(), getMessages().error_localitzacio_obligatoria(), null);
			}			
		}
	}

	@Override
	public void onDelete(XCS deleteObject) {
		if (deleteObject != null) {
			// Intentem esborrar el esdeveniment
			xService.delete(deleteObject, new AsyncCallback<Void>() {
				@Override
				public void onFailure(Throwable arg0) {
					// En cas de error mostrem un missatge generic de error
					MessageBox.alert(getMessages().dialeg_error(), getMessages().esborrar_ko(), null);
				}

				@Override
				public void onSuccess(Void arg0) {
					// En cas de esborrar-se correctament enviem un event al
					// grid perque
					// es recarregui
					getGrid().fireEvent(Events.Attach);
				}
			});
		}
	}

	/**
	 * Carrega el selector de productors disponibles.
	 * 
	 * @param idXcs
	 *            Id de la xcs.
	 */
	public void loadProductorsSource(Integer idXcs) {
		xService.getAllProductorsSource(idXcs, new AsyncCallback<List<Productor>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els productors disponibles", arg0);
			}

			@Override
			public void onSuccess(List<Productor> productors) {
				productorsSource.removeAll();
				productorsSource.add(productors);
			}
		});
	}

	/**
	 * Carrega el selector de productors associats.
	 * 
	 * @param idXcs
	 *            Id de la xcs.
	 */
	public void loadProductorsTarget(Integer idXcs) {
		xService.getAllProductorsTarget(idXcs, new AsyncCallback<List<Productor>>() {
			@Override
			public void onFailure(Throwable arg0) {
				GWT.log("Error al recuperar els productors seleccionats", arg0);
			}

			@Override
			public void onSuccess(List<Productor> productors) {
				productorsTarget.removeAll();
				productorsTarget.add(productors);
			}
		});
	}

	@Override
	public void onCancel() {
		this.getGrid().getSelectionModel().deselectAll();
		this.getFormPanel().clear();
		loadProductorsSource(null);
		loadProductorsTarget(null);
		comarca.clearSelections();
		poblacio.clearSelections();
		productors.setEnabled(false);
		mapaManteniment.setFeatureId(0);
	}
	
	@Override
	protected void onUnload() {
		onCancel();
		super.onUnload();
	}
	
	protected void refresh() {
		formAndMapPanel.remove(mapaManteniment);
		mapaManteniment = new MapaMantenimentLayout("xcs", "images/red-dot.png");		
		mapaManteniment.enableCreate();
		formAndMapPanel.add(mapaManteniment, new RowData(0.6, 1, new Margins(5, 5, 5, 5)));
		this.mapaManteniment.setHeading(getMessages().title_xcs_localitzacio());
		formAndMapPanel.layout();
		super.onAfterLayout();
	}

}
