package uoc.edu.client.service;

import java.util.List;

import uoc.edu.shared.model.Producte;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Servei web que gestiona els productes de l'aplicació.
 * 
 * @author Jordi Farré.
 * 
 */
@RemoteServiceRelativePath("producte")
public interface ProducteService extends RemoteService {

	/**
	 * Retorna tots els productes de l'aplicació.
	 * 
	 * @return Productes de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els productes.
	 */
	public List<Producte> getAll() throws IllegalArgumentException;

	/**
	 * Retorna tots els productes de l'aplicació.
	 * 
	 * @return Productes de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els productes.
	 */
	public PagingLoadResult<Producte> getAll(PagingLoadConfig config) throws IllegalArgumentException;

	/**
	 * Guarda un producte.
	 * 
	 * @param producte
	 *            Producte a guardar.
	 * @throws IllegalArgumentException
	 *             Excepció al guardar el producte.
	 */
	public void save(Producte producte) throws IllegalArgumentException;

	/**
	 * Esborra un producte.
	 * 
	 * @param producte
	 *            Producte a esborrar.
	 * @throws IllegalArgumentException
	 *             Error al esborrar el producte.
	 */
	public void delete(Producte producte) throws IllegalArgumentException;

}
