package uoc.edu.client.service;

import java.util.List;

import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Producte;
import uoc.edu.shared.model.Productor;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Servei web que gestiona els productor de l'aplicació.
 * 
 * @author Jordi Farré.
 * 
 */
@RemoteServiceRelativePath("productor")
public interface ProductorService extends RemoteService {

	/**
	 * Retorna tots els productors de l'aplicació.
	 * 
	 * @return productors de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els esdeveniments.
	 */
	public PagingLoadResult<Productor> getAll(PagingLoadConfig config) throws IllegalArgumentException;

	/**
	 * Esborra un productor.
	 * 
	 * @param productor
	 *            Productor a esborrar.
	 * @throws IllegalArgumentException
	 *             Error al esborrar el productor.
	 */
	public void delete(Productor productor) throws IllegalArgumentException;

	/**
	 * Retorna els esdeveniments disponibles per a un determinat productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Llista de esdeveniments disponibles.
	 */
	public List<Esdeveniment> getAllEsdevenimentsSource(Integer idProductor) throws IllegalArgumentException;

	/**
	 * Retorna tots els esdeveniments associats a un productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Esdeveniments asociats a un productor.
	 */
	public List<Esdeveniment> getAllEsdevenimentsTarget(Integer idProductor) throws IllegalArgumentException;

	/**
	 * Guarda els esdeveniments de un productor.
	 * 
	 * @param notSelected
	 *            Esdeveniments no seleccionats.
	 * @param selected
	 *            Esdeveniments seleccionats.
	 * @param idProductor
	 *            Identificador del productor.
	 * @throws IllegalArgumentException
	 *             Error al guardar els esdeveniments.
	 */
	public void saveEsdeveniments(List<Esdeveniment> notSelected, List<Esdeveniment> selected, Integer idProductor)
			throws IllegalArgumentException;

	/**
	 * Retorna els productes disponibles per a un determinat productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Llista de productes disponibles.
	 */
	public List<Producte> getAllProductesSource(Integer idProductor) throws IllegalArgumentException;

	/**
	 * Retorna tots els productes associats a un productor.
	 * 
	 * @param idProductor
	 *            Id del productor.
	 * @return Productes asociats a un productor.
	 */
	public List<Producte> getAllProductesTarget(Integer idProductor) throws IllegalArgumentException;

	/**
	 * Guarda els productes de un productor.
	 * 
	 * @param notSelected
	 *            Productes no seleccionats.
	 * @param selected
	 *            Productes seleccionats.
	 * @param idProductor
	 *            Identificador del productor.
	 * @throws IllegalArgumentException
	 *             Error al guardar els productes.
	 */
	public void saveProductes(List<Producte> notSelected, List<Producte> selected, Integer idProductor)
			throws IllegalArgumentException;

}
