package uoc.edu.client.service;

import java.util.List;

import uoc.edu.shared.model.TipusEsdeveniment;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Servei web que gestiona els tipus de esdeveniments de l'aplicació.
 * 
 * @author Jordi Farré.
 * 
 */
@RemoteServiceRelativePath("tipusesdeveniment")
public interface TipusEsdevenimentService extends RemoteService {
	
	/**
	 * Retorna tots els tipus de esdeveniments de l'aplicació.
	 * 
	 * @return Tipus de esdeveniments de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els tipus de esdeveniments.
	 */
	public List<TipusEsdeveniment> getAll() throws IllegalArgumentException;

	/**
	 * Retorna tots els tipus de esdeveniments de l'aplicació.
	 * 
	 * @return Tipus de esdeveniments de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els tipus de esdeveniments.
	 */
	public PagingLoadResult<TipusEsdeveniment> getAll(PagingLoadConfig config) throws IllegalArgumentException;

	/**
	 * Guarda un tipus de esdeveniment.
	 * 
	 * @param tipusEsdeveniment
	 *            Tipus de esdeveniment a guardar.
	 * @throws IllegalArgumentException
	 *             Excepció al guardar el tipus de esdeveniment.
	 */
	public void save(TipusEsdeveniment tipusEsdeveniment) throws IllegalArgumentException;

	/**
	 * Esborra un tipus de esdeveniment.
	 * @param tipusEsdeveniment Tipus de esdeveniment a esborrar.
	 * @throws IllegalArgumentException Error al esborrar el tipus de esdeveniment.
	 */
	public void delete(TipusEsdeveniment tipusEsdeveniment) throws IllegalArgumentException;

}
