package uoc.edu.client.service;

import java.util.List;

import uoc.edu.shared.model.TipusProductor;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Servei web que gestiona els tipus de productors de l'aplicació.
 * 
 * @author Jordi Farré.
 * 
 */
@RemoteServiceRelativePath("tipusproductor")
public interface TipusProductorService extends RemoteService {
	
	/**
	 * Retorna tots els tipus de productors de l'aplicació.
	 * 
	 * @return Tipus de productors de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els tipus de productors.
	 */
	public List<TipusProductor> getAll() throws IllegalArgumentException;

	/**
	 * Retorna tots els tipus de productors de l'aplicació.
	 * 
	 * @return Tipus de productors de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir els tipus de productors.
	 */
	public PagingLoadResult<TipusProductor> getAll(PagingLoadConfig config) throws IllegalArgumentException;

	/**
	 * Guarda un tipus de productor.
	 * 
	 * @param tipusProductor
	 *            Tipus de productor a guardar.
	 * @throws IllegalArgumentException
	 *             Excepció al guardar el tipus de productor.
	 */
	public void save(TipusProductor tipusProductor) throws IllegalArgumentException;

	/**
	 * Esborra un tipus de productor.
	 * @param tipusProductor Tipus de productor a esborrar.
	 * @throws IllegalArgumentException Error al esborrar el tipus de productor.
	 */
	public void delete(TipusProductor tipusProductor) throws IllegalArgumentException;

}
