package uoc.edu.client.service;

import java.util.List;

import uoc.edu.shared.model.Productor;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;
import com.google.gwt.user.client.rpc.RemoteService;
import com.google.gwt.user.client.rpc.RemoteServiceRelativePath;

/**
 * Servei web que gestiona les xcs de l'aplicació.
 * 
 * @author Jordi Farré.
 * 
 */
@RemoteServiceRelativePath("xcs")
public interface XcsService extends RemoteService {
	
	/**
	 * Retorna tots les xcs de l'aplicació.
	 * 
	 * @return xcs de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir les xcs.
	 */
	public List<XCS> getAll() throws IllegalArgumentException;

	/**
	 * Retorna tots les xcs de l'aplicació.
	 * 
	 * @return xcs de l'aplicació.
	 * @throws IllegalArgumentException
	 *             Error al obtenir les xcs.
	 */
	public PagingLoadResult<XCS> getAll(PagingLoadConfig config) throws IllegalArgumentException;

	/**
	 * Esborra una xcs.
	 * 
	 * @param xcs
	 *            XCS a esborrar.
	 * @throws IllegalArgumentException
	 *             Error al esborrar la xcs.
	 */
	public void delete(XCS xcs) throws IllegalArgumentException;

	/**
	 * Retorna els productors disponibles per a una determinada xcs.
	 * 
	 * @param idXcs
	 *            Id del xcs.
	 * @return Llista de productors disponibles.
	 */
	public List<Productor> getAllProductorsSource(Integer idXcs) throws IllegalArgumentException;

	/**
	 * Retorna tots els productors associats a una xcs.
	 * 
	 * @param idXcs
	 *            Id de la xcs.
	 * @return Productors asociats a un productor.
	 */
	public List<Productor> getAllProductorsTarget(Integer idXcs) throws IllegalArgumentException;

	/**
	 * Guarda els productors de una xcs.
	 * 
	 * @param notSelected
	 *            Productors no seleccionats.
	 * @param selected
	 *            Productors seleccionats.
	 * @param idXcs
	 *            Identificador de la xcs.
	 * @throws IllegalArgumentException
	 *             Error al guardar els productors.
	 */
	public void saveProductors(List<Productor> notSelected, List<Productor> selected, Integer idXcs)
			throws IllegalArgumentException;

}
