package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Comarca;

/**
 * DAO per el accés a la base de dades per la taula Comarca.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class ComarcaDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna totes les comarques de la base de dades.
	 * @return Totes les comarques de la base de dades.
	 */
	public List<Comarca> getAll() {
		String sql = "select idcomarca, descripcio from comarca";
		return this.template.query(sql, new RowMapper<Comarca>() {
			@Override
			public Comarca mapRow(ResultSet rs, int rowNum) throws SQLException {
				Comarca comarca = new Comarca();
				comarca.setId(rs.getInt("idcomarca"));
				comarca.setDescripcio(rs.getString("descripcio"));
				return comarca;
			}
		});
	}

}
