package uoc.edu.server.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Repository;

import uoc.edu.shared.model.Comarca;
import uoc.edu.shared.model.Poblacio;

/**
 * DAO per el accés a la base de dades per la taula Poblacio.
 * 
 * @author Jordi Farré
 * 
 */
@Repository
public class PoblacioDAO {

	/** Instancia de JDBC template */
	private JdbcTemplate template;

	/**
	 * Assigna el datasource i crea el jdbd template.
	 * 
	 * @param dataSource
	 *            Instancia del datasource.
	 */
	@Autowired
	public void setDataSource(DataSource dataSource) {
		template = new JdbcTemplate(dataSource);
	}

	/**
	 * Retorna totes les poblacions de la base de dades.
	 * @return Totes les poblacions de la base de dades.
	 */
	public List<Poblacio> getAll() {
		String sql = "select idpoblacio, codipostal poblaciocodipostal, descripcio poblaciodesc from poblacio order by poblaciodesc asc";
		return this.template.query(sql, new RowMapper<Poblacio>() {
			@Override
			public Poblacio mapRow(ResultSet rs, int rowNum) throws SQLException {
				Poblacio poblacio = new Poblacio();
				poblacio.setId(rs.getInt("idpoblacio"));
				poblacio.setCodiPostal(rs.getInt("poblaciocodipostal"));
				poblacio.setDescripcio(rs.getString("poblaciodesc"));
				return poblacio;
			}
		});
	}

}
