package uoc.edu.server.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import uoc.edu.client.service.ProductorService;
import uoc.edu.server.dao.ProductorDAO;
import uoc.edu.server.dao.ProductorEsdevenimentDAO;
import uoc.edu.server.dao.ProductorProducteDAO;
import uoc.edu.shared.model.Esdeveniment;
import uoc.edu.shared.model.Producte;
import uoc.edu.shared.model.Productor;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * Servei web que dona accés a les dades de productors.
 * 
 * @author Jordi Farré
 * 
 */
public class ProductorServiceImpl extends SpringGwtServlet implements ProductorService {

	/** DAO de la taula Productor */
	@Autowired
	private ProductorDAO productorDAO;

	/** DAO de la taula Productor_Esdeveniment */
	@Autowired
	private ProductorEsdevenimentDAO productorEsdevenimentDAO;

	/** DAO de la taula PRODUCTOR_PRODUCTE */
	@Autowired
	private ProductorProducteDAO productorProducteDAO;

	@Override
	public PagingLoadResult<Productor> getAll(PagingLoadConfig config) throws IllegalArgumentException {
		return productorDAO.getAll(config);
	}

	@Override
	public void delete(Productor productor) throws IllegalArgumentException {
		productorDAO.delete(productor);
	}

	/**
	 * @return the productorDAO
	 */
	public ProductorDAO getProductorDAO() {
		return productorDAO;
	}

	/**
	 * @param productorDAO
	 *            the productorDAO to set
	 */
	public void setProductorDAO(ProductorDAO productorDAO) {
		this.productorDAO = productorDAO;
	}

	/**
	 * @return the productorEsdevenimentDAO
	 */
	public ProductorEsdevenimentDAO getProductorEsdevenimentDAO() {
		return productorEsdevenimentDAO;
	}

	/**
	 * @param productorEsdevenimentDAO
	 *            the productorEsdevenimentDAO to set
	 */
	public void setProductorEsdevenimentDAO(ProductorEsdevenimentDAO productorEsdevenimentDAO) {
		this.productorEsdevenimentDAO = productorEsdevenimentDAO;
	}

	@Override
	public List<Esdeveniment> getAllEsdevenimentsSource(Integer idProductor) throws IllegalArgumentException {
		return this.productorEsdevenimentDAO.getAllSource(idProductor);
	}

	@Override
	public List<Esdeveniment> getAllEsdevenimentsTarget(Integer idProductor) throws IllegalArgumentException {
		return this.productorEsdevenimentDAO.getAllTarget(idProductor);
	}

	@Override
	public void saveEsdeveniments(List<Esdeveniment> notSelected, List<Esdeveniment> selected, Integer idProductor)
			throws IllegalArgumentException {
		// Primer mirem si els esdeveniments no seleccionats estan a bbdd, si ho
		// estan els esborrem
		for (Esdeveniment esdeveniment : notSelected) {
			if (productorEsdevenimentDAO.exists(idProductor, esdeveniment.getId())) {
				productorEsdevenimentDAO.delete(idProductor, esdeveniment.getId());
			}
		}
		// Ara mirem si els esdeveniments seleccionats estan a bbdd, si no ho
		// estan els creem
		for (Esdeveniment esdeveniment : selected) {
			if (!productorEsdevenimentDAO.exists(idProductor, esdeveniment.getId())) {
				productorEsdevenimentDAO.create(idProductor, esdeveniment.getId());
			}
		}
	}

	/**
	 * @return the productorProducteDAO
	 */
	public ProductorProducteDAO getProductorProducteDAO() {
		return productorProducteDAO;
	}

	/**
	 * @param productorProducteDAO
	 *            the productorProducteDAO to set
	 */
	public void setProductorProducteDAO(ProductorProducteDAO productorProducteDAO) {
		this.productorProducteDAO = productorProducteDAO;
	}

	@Override
	public List<Producte> getAllProductesSource(Integer idProductor) throws IllegalArgumentException {
		return this.productorProducteDAO.getAllSource(idProductor);
	}

	@Override
	public List<Producte> getAllProductesTarget(Integer idProductor) throws IllegalArgumentException {
		return this.productorProducteDAO.getAllTarget(idProductor);
	}

	@Override
	public void saveProductes(List<Producte> notSelected, List<Producte> selected, Integer idProductor)
			throws IllegalArgumentException {
		// Primer mirem si els productes no seleccionats estan a bbdd, si ho
		// estan els esborrem
		for (Producte producte : notSelected) {
			if (productorProducteDAO.exists(idProductor, producte.getId())) {
				productorProducteDAO.delete(idProductor, producte.getId());
			}
		}
		// Ara mirem si els productes seleccionats estan a bbdd, si no ho
		// estan els creem
		for (Producte producte : selected) {
			if (!productorProducteDAO.exists(idProductor, producte.getId())) {
				productorProducteDAO.create(idProductor, producte.getId());
			}
		}
	}

}
