package uoc.edu.server.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;

import uoc.edu.client.service.XcsService;
import uoc.edu.server.dao.XcsDAO;
import uoc.edu.server.dao.XcsProductorDAO;
import uoc.edu.shared.model.Productor;
import uoc.edu.shared.model.XCS;

import com.extjs.gxt.ui.client.data.PagingLoadConfig;
import com.extjs.gxt.ui.client.data.PagingLoadResult;

/**
 * Servei web que dona accés a les dades de xcs.
 * 
 * @author Jordi Farré
 * 
 */
public class XcsServiceImpl extends SpringGwtServlet implements XcsService {

	/** DAO de la taula Productor */
	@Autowired
	private XcsDAO xcsDAO;

	/** DAO de la taula Productor_Esdeveniment */
	@Autowired
	private XcsProductorDAO xcsProductorDAO;

	@Override
	public PagingLoadResult<XCS> getAll(PagingLoadConfig config) throws IllegalArgumentException {
		return xcsDAO.getAll(config);
	}

	@Override
	public void delete(XCS xcs) throws IllegalArgumentException {
		xcsDAO.delete(xcs);
	}

	@Override
	public List<Productor> getAllProductorsSource(Integer idXcs) throws IllegalArgumentException {
		return this.xcsProductorDAO.getAllSource(idXcs);
	}

	@Override
	public List<Productor> getAllProductorsTarget(Integer idXcs) throws IllegalArgumentException {
		return this.xcsProductorDAO.getAllTarget(idXcs);
	}

	@Override
	public void saveProductors(List<Productor> notSelected, List<Productor> selected, Integer idXcs)
			throws IllegalArgumentException {
		// Primer mirem si els productors no seleccionats estan a bbdd, si ho
		// estan els esborrem
		for (Productor productor : notSelected) {
			if (xcsProductorDAO.exists(idXcs, productor.getId())) {
				xcsProductorDAO.delete(idXcs, productor.getId());
			}
		}
		// Ara mirem si els productors seleccionats estan a bbdd, si no ho
		// estan els creem
		for (Productor productor : selected) {
			if (!xcsProductorDAO.exists(idXcs, productor.getId())) {
				xcsProductorDAO.create(idXcs, productor.getId());
			}
		}
	}

	/**
	 * @return the xcsDAO
	 */
	public XcsDAO getXcsDAO() {
		return xcsDAO;
	}

	/**
	 * @param xcsDAO
	 *            the xcsDAO to set
	 */
	public void setXcsDAO(XcsDAO xcsDAO) {
		this.xcsDAO = xcsDAO;
	}

	/**
	 * @return the xcsProductorDAO
	 */
	public XcsProductorDAO getXcsProductorDAO() {
		return xcsProductorDAO;
	}

	/**
	 * @param xcsProductorDAO
	 *            the xcsProductorDAO to set
	 */
	public void setXcsProductorDAO(XcsProductorDAO xcsProductorDAO) {
		this.xcsProductorDAO = xcsProductorDAO;
	}

	@Override
	public List<XCS> getAll() throws IllegalArgumentException {
		return this.xcsDAO.getAll();
	}

}
